@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 27824 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2012-06-14 08:58:20 +1000 (Thu, 14 Jun 2012) $
  */
 
 #ifndef PTLIB_HTTPFORM_H
@@ -44,7 +44,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // PHTTPField
 
-/** This class is the abstract base class for fields in a #PHTTPForm#
+/** This class is the abstract base class for fields in a <code>PHTTPForm</code>
    resource type.
  */
 class PHTTPField : public PObject
@@ -163,7 +163,7 @@ class PHTTPField : public PObject
        @return
        String for field value.
      */
-    virtual PString GetValue(PBoolean dflt = PFalse) const = 0;
+    virtual PString GetValue(PBoolean dflt = false) const = 0;
 
     /** Set the value of the field.
      */
@@ -185,7 +185,7 @@ class PHTTPField : public PObject
       PConfig & cfg   // Configuration for value transfer.
     ) const;
 
-    /** Validate the new field value before #SetValue()# is called.
+    /** Validate the new field value before <code>SetValue()</code> is called.
 
        @return
        PBoolean if the new field value is OK.
@@ -209,7 +209,7 @@ class PHTTPField : public PObject
       const PStringToString & data   // New value for the field.
     );
 
-    /** Validate the new field value in a list before #SetValue()# is called.
+    /** Validate the new field value in a list before <code>SetValue()</code> is called.
 
        @return
        PBoolean if the all the new field values are OK.
@@ -221,7 +221,7 @@ class PHTTPField : public PObject
 
 
     PBoolean NotYetInHTML() const { return notInHTML; }
-    void SetInHTML() { notInHTML = PFalse; }
+    void SetInHTML() { notInHTML = false; }
 
   protected:
     PCaselessString baseName;
@@ -241,7 +241,8 @@ class PHTTPCompositeField : public PHTTPField
     PHTTPCompositeField(
       const char * name,          // Name (identifier) for the field.
       const char * title = NULL,  // Title text for field (defaults to name).
-      const char * help = NULL    // Help text for the field.
+      const char * help = NULL,   // Help text for the field.
+      bool includeHeaders = false // Make a sub-table and put headers on HTML fields.
     );
 
     virtual void SetName(
@@ -268,7 +269,7 @@ class PHTTPCompositeField : public PHTTPField
       PHTML & html    // HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newValue   // New value for the field.
@@ -308,6 +309,7 @@ class PHTTPCompositeField : public PHTTPField
 
   protected:
     PHTTPFields fields;
+    bool        m_includeHeaders;
 };
 
 
@@ -396,16 +398,20 @@ class PHTTPStringField : public PHTTPField
   public:
     PHTTPStringField(
       const char * name,
-      PINDEX size,
+      PINDEX maxLength,
       const char * initVal = NULL,
-      const char * help = NULL
+      const char * help = NULL,
+      int rows = 0, // 0=auto, 1=InputText, >1=TextArea
+      int columns = 0
     );
     PHTTPStringField(
       const char * name,
       const char * title,
-      PINDEX size,
+      PINDEX maxLength,
       const char * initVal = NULL,
-      const char * help = NULL
+      const char * help = NULL,
+      int rows = 0, // 0=auto, 1=InputText, >1=TextArea
+      int columns = 0
     );
 
     virtual PHTTPField * NewField() const;
@@ -414,7 +420,7 @@ class PHTTPStringField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -424,7 +430,9 @@ class PHTTPStringField : public PHTTPField
   protected:
     PString value;
     PString initialValue;
-    PINDEX size;
+    PINDEX maxLength;
+    int rows;
+    int columns;
 };
 
 
@@ -452,7 +460,7 @@ class PHTTPPasswordField : public PHTTPStringField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -462,6 +470,32 @@ class PHTTPPasswordField : public PHTTPStringField
 };
 
 
+class PHTTPDateField : public PHTTPStringField
+{
+  PCLASSINFO(PHTTPDateField, PHTTPStringField)
+  public:
+    PHTTPDateField(
+      const char * name,
+      const PTime & initVal = PTime(0),
+      PTime::TimeFormat fmt = PTime::ShortDate
+    );
+
+    virtual PHTTPField * NewField() const;
+
+    virtual void SetValue(
+      const PString & newValue
+    );
+
+    virtual PBoolean Validated(
+      const PString & newValue,
+      PStringStream & msg
+    ) const;
+
+  protected:
+    PTime::TimeFormat m_format;
+};
+
+
 class PHTTPIntegerField : public PHTTPField
 {
   PCLASSINFO(PHTTPIntegerField, PHTTPField)
@@ -488,7 +522,7 @@ class PHTTPIntegerField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -520,13 +554,13 @@ class PHTTPBooleanField : public PHTTPField
   public:
     PHTTPBooleanField(
       const char * name,
-      PBoolean initVal = PFalse,
+      PBoolean initVal = false,
       const char * help = NULL
     );
     PHTTPBooleanField(
       const char * name,
       const char * title,
-      PBoolean initVal = PFalse,
+      PBoolean initVal = false,
       const char * help = NULL
     );
 
@@ -540,7 +574,7 @@ class PHTTPBooleanField : public PHTTPField
       const PString & input
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -634,7 +668,7 @@ class PHTTPRadioField : public PHTTPField
       const PString & input
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
@@ -657,21 +691,24 @@ class PHTTPSelectField : public PHTTPField
       const char * name,
       const PStringArray & valueArray,
       PINDEX initVal = 0,
-      const char * help = NULL
+      const char * help = NULL,
+      bool enumeration = false
     );
     PHTTPSelectField(
       const char * name,
       PINDEX count,
       const char * const * valueStrings,
       PINDEX initVal = 0,
-      const char * help = NULL
+      const char * help = NULL,
+      bool enumeration = false
     );
     PHTTPSelectField(
       const char * name,
       const char * title,
       const PStringArray & valueArray,
       PINDEX initVal = 0,
-      const char * help = NULL
+      const char * help = NULL,
+      bool enumeration = false
     );
     PHTTPSelectField(
       const char * name,
@@ -679,7 +716,8 @@ class PHTTPSelectField : public PHTTPField
       PINDEX count,
       const char * const * valueStrings,
       PINDEX initVal = 0,
-      const char * help = NULL
+      const char * help = NULL,
+      bool enumeration = false
     );
 
     virtual PHTTPField * NewField() const;
@@ -688,19 +726,55 @@ class PHTTPSelectField : public PHTTPField
       PHTML & html    ///< HTML to receive the field info.
     ) const;
 
-    virtual PString GetValue(PBoolean dflt = PFalse) const;
+    virtual PString GetValue(PBoolean dflt = false) const;
 
     virtual void SetValue(
       const PString & newVal
     );
 
-
-    PStringArray values;
-
-
   protected:
-    PString value;
-    PINDEX initialValue;
+    PStringArray m_values;
+    bool         m_enumeration;
+    PINDEX       m_initialValue;
+    PString      m_value;
+};
+
+
+template <typename Enumeration, Enumeration InitVal = (Enumeration)0>
+class PHTTPEnumField : public PHTTPSelectField
+{
+    PCLASSINFO(PHTTPEnumField, PHTTPSelectField)
+  public:
+    PHTTPEnumField(
+      const char * name,
+      const PStringArray & valueArray,
+      Enumeration initVal = InitVal,
+      const char * help = NULL
+    ) : PHTTPSelectField(name, valueArray, initVal, help, true) { }
+    PHTTPEnumField(
+      const char * name,
+      PINDEX count,
+      const char * const * valueStrings,
+      Enumeration initVal = InitVal,
+      const char * help = NULL
+    ) : PHTTPSelectField(name, count, valueStrings, initVal, help, true) { }
+    PHTTPEnumField(
+      const char * name,
+      const char * title,
+      const PStringArray & valueArray,
+      Enumeration initVal = InitVal,
+      const char * help = NULL
+    ) : PHTTPSelectField(name, title, valueArray, initVal, help, true) { }
+    PHTTPEnumField(
+      const char * name,
+      const char * title,
+      PINDEX count,
+      const char * const * valueStrings,
+      Enumeration initVal = InitVal,
+      const char * help = NULL
+    ) : PHTTPSelectField(name, title, count, valueStrings, initVal, help, true) { }
+
+    virtual PHTTPField * NewField() const { return new PHTTPEnumField(baseName, title, m_values, (Enumeration)m_initialValue, help); }
 };
 
 
