@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26516 $
+ * $Revision: 26840 $
  * $Author: rjongbloed $
- * $Date: 2011-10-04 18:28:25 -0500 (Tue, 04 Oct 2011) $
+ * $Date: 2012-01-08 18:22:58 +1100 (Sun, 08 Jan 2012) $
  */
 
 #ifndef PTLIB_PMIME_H
@@ -35,11 +35,11 @@
 #pragma interface
 #endif
 
-#include <ptclib/inetprot.h>
 #include <ptclib/cypher.h>
 
 
 class PMultiPartList;
+class PInternetProtocol;
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -61,6 +61,12 @@ class PMIMEInfo : public PStringOptions
     PMIMEInfo(
       PInternetProtocol & socket   ///< Application socket to read MIME info.
     );
+    PMIMEInfo(
+      const PStringToString & dict
+    );
+    PMIMEInfo(
+      const PString & str
+    );
 
 
   // Overrides from class PObject
@@ -98,6 +104,10 @@ class PMIMEInfo : public PStringOptions
       PInternetProtocol & socket   ///< Application socket to write MIME info.
     ) const;
 
+    /**Return a string presentation of the MIME.
+      */
+    PString AsString() const;
+
     /**Add a MIME field given a "name: value" format string.
        Note that if the field name was already in the MIME dictionary then
        this will append the new value after a '\\n' character to the previous
