@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22443 $
- * $Author: rjongbloed $
- * $Date: 2009-04-20 18:47:22 -0500 (Mon, 20 Apr 2009) $
+ * $Revision: 25539 $
+ * $Author: willamowius $
+ * $Date: 2011-04-08 22:26:56 +1000 (Fri, 08 Apr 2011) $
  */
 
 #ifndef PTLIB_PLDAP_H
@@ -111,6 +111,10 @@ class PLDAPSession : public PObject
 #endif
     };
 
+    /**Start encrypted connection
+      */
+    PBoolean StartTLS();
+
     /**Bind to the remote LDAP server.
       */
     PBoolean Bind(
@@ -181,7 +185,7 @@ class PLDAPSession : public PObject
         virtual void SetLDAPModVars(struct ldapmod & mod);
 
         PStringArray values;
-        PBaseArray<char *> pointers;
+        PBaseArray<const char *> pointers;
     };
 
     class BinaryModAttrib : public ModAttrib {
@@ -467,8 +471,9 @@ class PLDAPStructBase : public PObject {
   public:
     void PrintOn(ostream & strm) const;
 
-    PINDEX GetNumAttributes() const { return attributes.GetSize(); }
-    PLDAPAttributeBase & GetAttribute(PINDEX idx) const { return attributes.GetDataAt(idx); }
+    typedef PDictionary<PString, PLDAPAttributeBase> AttribDict;
+    AttribDict & GetAttributes() { return attributes; }
+    const AttribDict & GetAttributes() const { return attributes; }
     PLDAPAttributeBase * GetAttribute(const char * name) const { return attributes.GetAt(name); }
 
     void AddAttribute(PLDAPAttributeBase * var);
@@ -477,7 +482,7 @@ class PLDAPStructBase : public PObject {
   protected:
     void EndConstructor();
 
-    PDictionary<PString, PLDAPAttributeBase> attributes;
+    AttribDict attributes;
 
     PLDAPStructBase        * initialiserStack;
     static PMutex            initialiserMutex;
