@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25319 $
+ * $Revision: 29661 $
  * $Author: rjongbloed $
- * $Date: 2011-03-14 20:33:53 -0500 (Mon, 14 Mar 2011) $
+ * $Date: 2013-05-08 15:02:01 +1000 (Wed, 08 May 2013) $
  */
 
 #ifndef PTLIB_PURL_H
@@ -41,15 +41,21 @@
 #include <ptlib/pfactory.h>
 
 
+class PURLScheme;
+class PURLLegacyScheme;
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PURL
 
-class PURLLegacyScheme;
-
 /**
  This class describes a Universal Resource Locator.
  This is the desciption of a resource location as used by the World Wide
  Web and the <code>PHTTPSocket</code> class.
+
+ Note, this has been extended to be for URI's (Universal Resource  Identifier)
+ which is a superset of URL's. But for backward compatinility reasons the name
+ is still PURL.
  */
 class PURL : public PObject
 {
@@ -129,14 +135,14 @@ class PURL : public PObject
 
     /**Print/String output representation formats. */
     enum UrlFormat {
-      /// Translate to a string as a full URL
-      FullURL,      
-      /// Translate to a string as only path
-      PathOnly,     
-      /// Translate to a string with no scheme or host
-      URIOnly,      
-      /// Translate to a string with scheme and host/port
-      HostPortOnly  
+      FullURL,       ///< Output full URI
+      PathOnly,      ///< Translate to a string as only path
+      LocationOnly,  ///< Translate to a string with the location (scheme and user/pass/host/port)
+      RelativeOnly,  ///< Translate to a string with no scheme or host, just the relative part
+
+      // for backward compatibility
+      HostPortOnly = LocationOnly,
+      URIOnly = RelativeOnly
     };
 
     /**Convert the URL object into its string representation. The parameter
@@ -164,7 +170,9 @@ class PURL : public PObject
       /// Translate the query variable field for a URL.
       QueryTranslation,
       /// Translate the parameter variables field for a URL.
-      ParameterTranslation
+      ParameterTranslation,
+      /// Translate the quoted parameter variables field for a URL.
+      QuotedParameterTranslation
     };
 
     /**Translate a string from general form to one that can be included into
@@ -222,7 +230,7 @@ class PURL : public PObject
     const PCaselessString & GetScheme() const { return scheme; }
 
     /// Set the scheme field of the URL
-    void SetScheme(const PString & scheme);
+    bool SetScheme(const PString & scheme);
 
     /// Get the username field of the URL.
     const PString & GetUserName() const { return username; }
@@ -245,12 +253,15 @@ class PURL : public PObject
     /// Get the port field of the URL.
     WORD GetPort() const { return port; }
 
-    /// Set the port field in the URL.
+    /// Set the port field in the URL. Zero resets to default.
     void SetPort(WORD newPort);
     
     /// Get if explicit port is specified.
     PBoolean GetPortSupplied() const { return portSupplied; }
 
+    /// Get the hostname and optional port fields of the URL.
+    PString GetHostPort() const;
+
     /// Get if path is relative or absolute
     PBoolean GetRelativePath() const { return relativePath; }
 
@@ -269,6 +280,12 @@ class PURL : public PObject
     /// Append segment to the path field of the URL.
     void AppendPath(const PString & segment);
 
+    /// Change segment in the path field of the URL.
+    void ChangePath(
+      const PString & segment, ///< New value for segment, empty means remove
+      PINDEX idx = P_MAX_INDEX ///< Segment index, P_MAX_INDEX means last segment
+    );
+
     /// Get the parameter (;) field of the URL.
     PString GetParameters() const;
 
@@ -277,7 +294,7 @@ class PURL : public PObject
 
     /// Get the parameter (;) field(s) of the URL as a string dictionary.
     /// Note the values have already been translated using UntranslateString
-    const PStringToString & GetParamVars() const { return paramVars; }
+    const PStringOptions & GetParamVars() const { return paramVars; }
 
     /// Set the parameter (;) field(s) of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
@@ -303,7 +320,7 @@ class PURL : public PObject
 
     /// Get the Query (?) field of the URL as a string dictionary.
     /// Note the values have already been translated using UntranslateString
-    const PStringToString & GetQueryVars() const { return queryVars; }
+    const PStringOptions & GetQueryVars() const { return queryVars; }
 
     /// Set the Query (?) field(s) of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
@@ -323,18 +340,46 @@ class PURL : public PObject
     PBoolean IsEmpty() const { return urlString.IsEmpty(); }
 
 
+    struct LoadParams {
+      LoadParams(
+        const PString & requiredContentType = PString::Empty(),
+        const PTimeInterval & timeout = PMaxTimeInterval
+      ) : m_requiredContentType(requiredContentType)
+        , m_timeout(timeout)
+      {
+      }
+
+      PString       m_requiredContentType;
+      PTimeInterval m_timeout;
+
+      PString       m_username;     // Basic authentication
+      PString       m_password;
+#if P_SSL
+      PString       m_authority;    // Directory, file or data
+      PString       m_certificate;  // File or data
+      PString       m_privateKey;   // File or data
+#endif
+    };
     /**Get the resource the URL is pointing at.
        The data returned is obtained according to the scheme and the factory
        PURLLoaderFactory.
       */
     bool LoadResource(
       PString & data,  ///< Resource data as a string
-      const PString & requiredContentType = PString::Empty() ///< Expected content type where applicable
-    );
+      const LoadParams & params = LoadParams()  ///< Parameters for load
+    ) const;
     bool LoadResource(
       PBYTEArray & data,  ///< Resource data as a binary blob
-      const PString & requiredContentType = PString::Empty() ///< Expected content type where applicable
-    );
+      const LoadParams & params = LoadParams()  ///< Parameters for load
+    ) const;
+
+    // For backward compatibility
+    template <class T>
+    bool LoadResource(
+      T & data,  ///< Resource data as a string
+      const PString & requiredContentType = PString::Empty(), ///< Expected content type where applicable
+      const PTimeInterval & timeout = PMaxTimeInterval        ///< Timeout to wait for resource
+    ) const { return LoadResource(data, LoadParams(requiredContentType, timeout)); }
 
     /**Open the URL in a browser.
 
@@ -348,7 +393,7 @@ class PURL : public PObject
     );
   //@}
 
-    PBoolean LegacyParse(const PString & url, const PURLLegacyScheme * schemeInfo);
+    bool LegacyParse(const char * str, const PURLLegacyScheme * schemeInfo);
     PString LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schemeInfo) const;
 
   protected:
@@ -358,20 +403,22 @@ class PURL : public PObject
       const char * defaultScheme ///< Default scheme for URL
     );
     void Recalculate();
+
+    const PURLScheme * schemeInfo;
     PString urlString;
 
     PCaselessString scheme;
-    PString username;
-    PString password;
+    PString         username;
+    PString         password;
     PCaselessString hostname;
-    WORD port;
-    PBoolean portSupplied;          /// port was supplied in string input
-    PBoolean relativePath;
-    PStringArray path;
-    PStringToString paramVars;
-    PString fragment;
-    PStringToString queryVars;
-    PString m_contents;  // Anything left after parsing other elements
+    WORD            port;
+    bool            portSupplied;          /// port was supplied in string input
+    bool            relativePath;
+    PStringArray    path;
+    PStringOptions paramVars;
+    PString         fragment;
+    PStringOptions queryVars;
+    PString         m_contents;  // Anything left after parsing other elements
 };
 
 
@@ -382,9 +429,9 @@ class PURLScheme : public PObject
 {
   PCLASSINFO(PURLScheme, PObject);
   public:
-    virtual PString GetName() const = 0;
-    virtual PBoolean Parse(const PString & url, PURL & purl) const = 0;
+    virtual bool Parse(const char * cstr, PURL & url) const = 0;
     virtual PString AsString(PURL::UrlFormat fmt, const PURL & purl) const = 0;
+    virtual WORD GetDefaultPort() const { return 0; }
 };
 
 typedef PFactory<PURLScheme> PURLSchemeFactory;
@@ -397,7 +444,6 @@ class PURLLegacyScheme : public PURLScheme
 {
   public:
     PURLLegacyScheme(
-      const char * s,
       bool user    = false,
       bool pass    = false,
       bool host    = false,
@@ -410,8 +456,7 @@ class PURLLegacyScheme : public PURLScheme
       bool rel     = false,
       WORD port    = 0
     )
-      : scheme(s)
-      , hasUsername           (user)
+      : hasUsername           (user)
       , hasPassword           (pass)
       , hasHostPort           (host)
       , defaultToUserIfNoAt   (def)
@@ -424,16 +469,18 @@ class PURLLegacyScheme : public PURLScheme
       , defaultPort           (port)
     { }
 
-    PBoolean Parse(const PString & url, PURL & purl) const
-    { return purl.LegacyParse(url, this); }
+    bool Parse(const char * cstr, PURL & url) const
+    {
+      return url.LegacyParse(cstr, this);
+    }
 
-    PString AsString(PURL::UrlFormat fmt, const PURL & purl) const
-    { return purl.LegacyAsString(fmt, this); }
+    PString AsString(PURL::UrlFormat fmt, const PURL & url) const
+    {
+      return url.LegacyAsString(fmt, this);
+    }
 
-    PString GetName() const     
-    { return scheme; }
+    virtual WORD GetDefaultPort() const { return defaultPort; }
 
-    PString scheme;
     bool hasUsername;
     bool hasPassword;
     bool hasHostPort;
@@ -447,12 +494,39 @@ class PURLLegacyScheme : public PURLScheme
     WORD defaultPort;
 };
 
-#define PURL_LEGACY_SCHEME(schemeName, user, pass, host, def, defhost, query, params, frags, path, rel, port) \
+/** Define a scheme based on basic legacy syntax.
+    The full HTTP style URL is:
+      scheme://user:pass@host:port/path#fragment;params?query
+    the various flags indicate if the subsection is present in the scheme.
+
+  */
+#define PURL_LEGACY_SCHEME(schemeName, \
+                           hasUsername,           /* URL scheme has a ysername */ \
+                           hasPassword,           /* URL scheme has a password */ \
+                           hasHostPort,           /* URL scheme has a host:port */ \
+                           defaultToUserIfNoAt,   /* URL scheme is username if no @, otherwise host:port */ \
+                           defaultHostToLocal,    /* URL scheme defaults to PIPSocket::GetHostName() if not present */ \
+                           hasQuery,              /* URL scheme has a query secton */ \
+                           hasParameters,         /* URL scheme has a parameter section */ \
+                           hasFragments,          /* URL scheme has a fragment section */ \
+                           hasPath,               /* URL scheme has a path */ \
+                           relativeImpliesScheme, /* URL scheme has relative path (no //) then scheme: is not output */ \
+                           defaultPort)           /* URL scheme default port if not specified in host:port */ \
   class PURLLegacyScheme_##schemeName : public PURLLegacyScheme \
   { \
     public: \
       PURLLegacyScheme_##schemeName() \
-        : PURLLegacyScheme(#schemeName, user, pass, host, def, defhost, query, params, frags, path, rel, port) \
+        : PURLLegacyScheme(hasUsername, \
+                           hasPassword, \
+                           hasHostPort, \
+                           defaultToUserIfNoAt, \
+                           defaultHostToLocal, \
+                           hasQuery, \
+                           hasParameters, \
+                           hasFragments, \
+                           hasPath, \
+                           relativeImpliesScheme, \
+                           defaultPort) \
         { } \
   }; \
   static PURLSchemeFactory::Worker<PURLLegacyScheme_##schemeName> schemeName##Factory(#schemeName, true); \
@@ -466,12 +540,19 @@ class PURLLoader : public PObject
 {
   PCLASSINFO(PURLLoader, PObject);
   public:
-    virtual bool Load(const PURL & url, PString & str, const PString & requiredContentType) = 0;
-    virtual bool Load(const PURL & url, PBYTEArray & data, const PString & requiredContentType) = 0;
+    virtual bool Load(PString & str, const PURL & url, const PURL::LoadParams & params) const = 0;
+    virtual bool Load(PBYTEArray & data, const PURL & url, const PURL::LoadParams & params) const = 0;
 };
 
 typedef PFactory<PURLLoader> PURLLoaderFactory;
 
+#if P_HTTP
+  PFACTORY_LOAD(PURL_HttpLoader);
+#endif // P_HTTP
+#if P_FTP
+  PFACTORY_LOAD(PURL_FtpLoader);
+#endif // P_HTTP
+
 
 #endif // P_URL
 
