@@ -21,9 +21,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 26456 $
+ * $Revision: 27706 $
  * $Author: rjongbloed $
- * $Date: 2011-09-22 00:27:10 -0500 (Thu, 22 Sep 2011) $
+ * $Date: 2012-05-25 18:23:39 +1000 (Fri, 25 May 2012) $
  */
 
 #ifndef PTLIB_VSDL_H
@@ -32,12 +32,9 @@
 #if P_SDL
 
 #include <ptlib.h>
-#include <SDL/SDL.h>
 
 #include <ptlib/videoio.h>
 
-#undef main
-
 
 /**Display data to the SDL screen.
   */
@@ -119,14 +116,16 @@ class PVideoOutputDevice_SDL : public PVideoOutputDevice
     );
 
   protected:
+    struct SDL_Overlay * m_overlay;
     PSyncPoint    m_operationComplete;
-    SDL_Overlay * m_overlay;
     unsigned      m_x, m_y;
 
   private:
     PString GetTitle() const;
     void UpdateContent();
-    void PostEvent(unsigned code);
+    void CreateOverlay(struct SDL_Surface * surface);
+    void FreeOverlay();
+    void PostEvent(unsigned codei, bool wait);
 
   friend class PSDL_Window;
 };
