@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28153 $
+ * $Revision: 30043 $
  * $Author: rjongbloed $
- * $Date: 2012-08-06 22:45:21 -0500 (Mon, 06 Aug 2012) $
+ * $Date: 2013-06-25 12:34:24 +1000 (Tue, 25 Jun 2013) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -81,6 +81,9 @@ class PVXMLGrammar : public PObject
 
     GrammarState GetState() const { return m_state; }
 
+    void SetSessionTimeout();
+    void SetTimeout(const PTimeInterval & timeout);
+
   protected:
     PDECLARE_NOTIFIER(PTimer, PVXMLGrammar, OnTimeout);
 
@@ -88,6 +91,7 @@ class PVXMLGrammar : public PObject
     PXMLElement  & m_field;
     PString        m_value;
     GrammarState   m_state;
+    PTimeInterval  m_timeout;
     PTimer         m_timer;
     PMutex         m_mutex;
 };
@@ -130,35 +134,40 @@ class PVXMLDigitsGrammar : public PVXMLGrammar
 
 //////////////////////////////////////////////////////////////////
 
-class PVXMLCache : public PMutex
+class PVXMLCache : public PSafeObject
 {
   public:
-    PVXMLCache(const PDirectory & directory);
+    PVXMLCache();
 
-    PFilePath CreateFilename(const PString & prefix, const PString & key, const PString & fileType);
+    virtual bool Get(
+      const PString & prefix,
+      const PString & key,
+      const PString & fileType,
+      PFilePath     & filename
+    );
 
-    void Put(const PString & prefix,
-             const PString & key, 
-             const PString & fileType, 
-             const PString & contentType,       
-           const PFilePath & fn, 
-                 PFilePath & dataFn);
+    virtual bool PutWithLock(
+      const PString   & prefix,
+      const PString   & key,
+      const PString   & fileType,
+      PFile           & file
+    );
 
-    PBoolean Get(const PString & prefix,
-             const PString & key, 
-             const PString & fileType, 
-                   PString & contentType,       
-                 PFilePath & fn);
+    void SetDirectory(
+      const PDirectory & directory
+    );
 
-    PFilePath GetCacheDir() const
-    { return directory; }
-
-    PFilePath GetRandomFilename(const PString & prefix, const PString & fileType);
-
-    static PVXMLCache & GetResourceCache();
+    const PDirectory & GetDirectory() const
+    { return m_directory; }
 
   protected:
-    PDirectory directory;
+    virtual PFilePath CreateFilename(
+      const PString & prefix,
+      const PString & key,
+      const PString & fileType
+    );
+
+    PDirectory m_directory;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -177,6 +186,12 @@ class PVXMLSession : public PIndirectChannel
     PTextToSpeech * SetTextToSpeech(const PString & ttsName);
     PTextToSpeech * GetTextToSpeech() const { return m_textToSpeech; }
 
+    void SetCache(PVXMLCache & cache);
+    PVXMLCache & GetCache();
+
+    void SetRecordDirectory(const PDirectory & dir) { m_recordDirectory = dir; }
+    const PDirectory & GetRecordDirectory() const { return m_recordDirectory; }
+
     virtual PBoolean Load(const PString & source);
     virtual PBoolean LoadFile(const PFilePath & file, const PString & firstForm = PString::Empty());
     virtual PBoolean LoadURL(const PURL & url);
@@ -195,7 +210,6 @@ class PVXMLSession : public PIndirectChannel
     virtual PBoolean LoadGrammar(PVXMLGrammar * grammar);
 
     virtual PBoolean PlayText(const PString & text, PTextToSpeech::TextType type = PTextToSpeech::Default, PINDEX repeat = 1, PINDEX delay = 0);
-    PBoolean ConvertTextToFilenameList(const PString & text, PTextToSpeech::TextType type, PStringArray & list, PBoolean useCacheing);
 
     virtual PBoolean PlayFile(const PString & fn, PINDEX repeat = 1, PINDEX delay = 0, PBoolean autoDelete = false);
     virtual PBoolean PlayData(const PBYTEArray & data, PINDEX repeat = 1, PINDEX delay = 0);
@@ -213,9 +227,6 @@ class PVXMLSession : public PIndirectChannel
     virtual void SetPause(PBoolean pause);
     virtual void GetBeepData(PBYTEArray & data, unsigned ms);
 
-    virtual PBoolean StartRecording(const PFilePath & fn, PBoolean recordDTMFTerm, const PTimeInterval & recordMaxTime, const PTimeInterval & recordFinalSilence);
-    virtual PBoolean EndRecording();
-
     virtual void OnUserInput(const PString & str);
 
     PString GetXMLError() const;
@@ -236,9 +247,7 @@ class PVXMLSession : public PIndirectChannel
     virtual void SetVar(const PString & ostr, const PString & val);
     virtual PString EvaluateExpr(const PString & oexpr);
 
-    static PTimeInterval StringToTime(const PString & str);
-
-    virtual PBoolean RetreiveResource(const PURL & url, PString & contentType, PFilePath & fn, PBoolean useCache = true);
+    static PTimeInterval StringToTime(const PString & str, int dflt = 0);
 
     PDECLARE_NOTIFIER(PThread, PVXMLSession, VXMLExecute);
 
@@ -246,7 +255,7 @@ class PVXMLSession : public PIndirectChannel
     bool GoToEventHandler(PXMLElement & element, const PString & eventName);
 
     // overrides from VXMLChannelInterface
-    virtual void OnEndRecording();
+    virtual void OnEndRecording(PINDEX bytesRecorded, bool timedOut);
     virtual void Trigger();
 
 
@@ -270,6 +279,7 @@ class PVXMLSession : public PIndirectChannel
     virtual PBoolean TraverseForm(PXMLElement & element);
     virtual PBoolean TraversedForm(PXMLElement & element);
     virtual PBoolean TraversePrompt(PXMLElement & element);
+    virtual PBoolean TraversedPrompt(PXMLElement & element);
     virtual PBoolean TraverseField(PXMLElement & element);
     virtual PBoolean TraversedField(PXMLElement & element);
     virtual PBoolean TraverseTransfer(PXMLElement & element);
@@ -278,6 +288,8 @@ class PVXMLSession : public PIndirectChannel
     __inline PVXMLChannel * GetVXMLChannel() const { return (PVXMLChannel *)readChannel; }
 
   protected:
+    virtual bool InternalLoadVXML(const PString & xml, const PString & firstForm);
+
     virtual bool ProcessNode();
     virtual bool ProcessEvents();
     virtual bool ProcessGrammar();
@@ -293,7 +305,8 @@ class PVXMLSession : public PIndirectChannel
     PURL             m_rootURL;
     PXML             m_xml;
 
-    PTextToSpeech *  m_textToSpeech;
+    PTextToSpeech  * m_textToSpeech;
+    PVXMLCache     * m_ttsCache;
     bool             m_autoDeleteTextToSpeech;
 
     PThread     *    m_vxmlThread;
@@ -302,6 +315,8 @@ class PVXMLSession : public PIndirectChannel
     PXMLObject  *    m_currentNode;
     bool             m_xmlChanged;
     bool             m_speakNodeData;
+    bool             m_bargeIn;
+    bool             m_bargingIn;
 
     PVXMLGrammar *   m_grammar;
     char             m_defaultMenuDTMF;
@@ -316,8 +331,11 @@ class PVXMLSession : public PIndirectChannel
       NotRecording,
       RecordingInProgress,
       RecordingComplete
-    }    m_recordingStatus;
-    bool m_recordStopOnDTMF;
+    }          m_recordingStatus;
+    bool       m_recordStopOnDTMF;
+    PString    m_recordingName;
+    PTime      m_recordingStartTime;
+    PDirectory m_recordDirectory;
 
     enum {
       NotTransfering,
@@ -344,23 +362,17 @@ class PVXMLRecordable : public PObject
 
     virtual PBoolean OnFrame(PBoolean /*isSilence*/) { return false; }
 
-    void SetFinalSilence(unsigned v)
-    { m_finalSilence = v > 0 ? v : 60000; }
+    const PTimeInterval & GetFinalSilence() const { return m_finalSilence; }
+    void SetFinalSilence(const PTimeInterval & v) { m_finalSilence = v > 0 ? v : 60000; }
 
-    unsigned GetFinalSilence()
-    { return m_finalSilence; }
-
-    void SetMaxDuration(unsigned v)
-    { m_maxDuration = v > 0 ? v : 86400000; }
-
-    unsigned GetMaxDuration()
-    { return m_maxDuration; }
+    const PTimeInterval & GetMaxDuration() const { return m_maxDuration; }
+    void SetMaxDuration(const PTimeInterval & v) { m_maxDuration = v > 0 ? v : 86400000; }
 
   protected:
-    PSimpleTimer m_silenceTimer;
-    PSimpleTimer m_recordTimer;
-    unsigned     m_finalSilence;
-    unsigned     m_maxDuration;
+    PSimpleTimer  m_silenceTimer;
+    PSimpleTimer  m_recordTimer;
+    PTimeInterval m_finalSilence;
+    PTimeInterval m_maxDuration;
 };
 
 //////////////////////////////////////////////////////////////////
@@ -457,6 +469,8 @@ class PVXMLPlayableData : public PVXMLPlayable
 
 //////////////////////////////////////////////////////////////////
 
+#if P_DTMF
+
 #include <ptclib/dtmf.h>
 
 class PVXMLPlayableTone : public PVXMLPlayableData
@@ -468,6 +482,9 @@ class PVXMLPlayableTone : public PVXMLPlayableData
     PTones m_tones;
 };
 
+#endif // P_DTMF
+
+
 //////////////////////////////////////////////////////////////////
 
 class PVXMLPlayableCommand : public PVXMLPlayable
@@ -534,8 +551,10 @@ class PVXMLChannel : public PDelayChannel
     virtual PBoolean Read(void * buffer, PINDEX amount);
     virtual PBoolean Write(const void * buf, PINDEX len);
 
+#if P_WAVFILE
     // new functions
     virtual PWAVFile * CreateWAVFile(const PFilePath & fn, PBoolean recording = false);
+#endif
 
     const PString & GetMediaFormat() const { return mediaFormat; }
     PBoolean IsMediaPCM() const { return mediaFormat == "PCM-16"; }
@@ -545,11 +564,9 @@ class PVXMLChannel : public PDelayChannel
     virtual PBoolean WriteFrame(const void * buf, PINDEX len) = 0;
     virtual PBoolean IsSilenceFrame(const void * buf, PINDEX len) const = 0;
 
-    virtual PBoolean QueueRecordable(PVXMLRecordable * newItem);
-
-    PBoolean StartRecording(const PFilePath & fn, unsigned finalSilence = 3000, unsigned maxDuration = 30000);
-    PBoolean EndRecording();
-    PBoolean IsRecording() const { return m_recordable != NULL; }
+    virtual bool QueueRecordable(PVXMLRecordable * newItem);
+    bool EndRecording(bool timedOut);
+    bool IsRecording() const { return m_recordable != NULL; }
 
     // Outgoing channel functions
     virtual PBoolean ReadFrame(void * buffer, PINDEX amount) = 0;
@@ -592,8 +609,6 @@ class PVXMLChannel : public PDelayChannel
 
     // Incoming audio variables
     PVXMLRecordable * m_recordable;
-    unsigned          m_finalSilence;
-    unsigned          m_silenceRun;
 
     // Outgoing audio variables
     PVXMLQueue      m_playQueue;
