@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27358 $
+ * $Revision: 29567 $
  * $Author: rjongbloed $
- * $Date: 2012-03-28 21:17:23 -0500 (Wed, 28 Mar 2012) $
+ * $Date: 2013-04-25 13:31:57 +1000 (Thu, 25 Apr 2013) $
  */
 
 
@@ -47,28 +47,32 @@
 /**This class is used when an ordinal index value is the key for <code>PSet</code>
    and <code>PDictionary</code> classes.
  */
-class POrdinalKey : public PObject
+template <typename T>
+class PKey : public PObject
 {
-  PCLASSINFO(POrdinalKey, PObject);
-
+    PCLASSINFO(PKey, PObject);
   public:
+    typedef T value_type;
+    typedef PKey<T> my_type;
+
   /**@name Construction */
   //@{
     /** Create a new key for ordinal index values.
      */
-    PINLINE POrdinalKey(
-      PINDEX newKey = 0   ///< Ordinal index value to use as a key.
-    );
+    PINLINE PKey(
+      value_type newKey = 0   ///< Ordinal index value to use as a key.
+    ) : m_key(newKey) { }
 
     /**Operator to assign the ordinal.
       */
-    PINLINE POrdinalKey & operator=(PINDEX);
+    PINLINE PKey & operator=(value_type newKey)
+    { this->m_key = newKey; return *this; }
   //@}
 
   /**@name Overrides from class PObject */
   //@{
-    /// Create a duplicate of the POrdinalKey.
-    virtual PObject * Clone() const;
+    /// Create a duplicate of the PKey.
+    virtual PObject * Clone() const { return new PKey(this->m_key); }
 
     /* Get the relative rank of the ordinal index. This is a simpel comparison
        of the objects PINDEX values.
@@ -79,7 +83,20 @@ class POrdinalKey : public PObject
        object and <code>GreaterThan</code> for \p obj logically
        greater than the object.
      */
-    virtual Comparison Compare(const PObject & obj) const;
+    virtual Comparison Compare(const PObject & obj) const
+    {
+      const my_type * other = dynamic_cast<const my_type *>(&obj);
+      if (!PAssert(other != NULL, PInvalidCast))
+        return GreaterThan;
+
+      if (this->m_key < other->m_key)
+        return LessThan;
+
+      if (this->m_key > other->m_key)
+        return GreaterThan;
+
+      return EqualTo;
+    }
 
     /**This function calculates a hash table index value for the implementation
        of <code>PSet</code> and <code>PDictionary</code> classes.
@@ -87,7 +104,7 @@ class POrdinalKey : public PObject
        @return
        hash table bucket number.
      */
-    virtual PINDEX HashFunction() const;
+    virtual PINDEX HashFunction() const { return PABSINDEX(this->m_key)%23; }
 
     /**Output the ordinal index to the specified stream. This is identical to
        outputting the PINDEX, i.e. integer, value.
@@ -95,44 +112,46 @@ class POrdinalKey : public PObject
        @return
        stream that the index was output to.
      */
-    virtual void PrintOn(ostream & strm) const;
+    virtual void PrintOn(ostream & strm) const { strm << this->m_key; }
   //@}
 
   /**@name New functions for class */
   //@{
-    /** Operator so that a POrdinalKey can be used as a PINDEX value.
+    /** Operator so that a PKey can be used as a PINDEX value.
      */
-    PINLINE operator PINDEX() const;
+    PINLINE operator value_type() const  { return this->m_key; }
 
     /**Operator to pre-increment the ordinal.
       */
-    PINLINE PINDEX operator++();
+    PINLINE value_type operator++() { return ++this->m_key; }
 
     /**Operator to post-increment the ordinal.
       */
-    PINLINE PINDEX operator++(int);
+    PINLINE value_type operator++(int) { return this->m_key++; }
 
     /**Operator to pre-decrement the ordinal.
       */
-    PINLINE PINDEX operator--();
+    PINLINE value_type operator--() { return --this->m_key; }
 
     /**Operator to post-decrement the ordinal.
       */
-    PINLINE PINDEX operator--(int);
+    PINLINE value_type operator--(int) { return this->m_key--; }
 
     /**Operator to add the ordinal.
       */
-    PINLINE POrdinalKey & operator+=(PINDEX);
+    PINLINE PKey & operator+=(value_type add) { this->m_key += add; return *this; }
 
     /**Operator to subtract from the ordinal.
       */
-    PINLINE POrdinalKey & operator-=(PINDEX );
+    PINLINE PKey & operator-=(value_type minus) { this->m_key -= minus; return *this; }
   //@}
 
   private:
-    PINDEX theKey;
+    value_type m_key;
 };
 
+typedef PKey<PINDEX> POrdinalKey;
+
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -143,23 +162,31 @@ struct PHashTableElement
     PObject * data;
     PHashTableElement * next;
     PHashTableElement * prev;
+    PINDEX bucket;
 
     PDECLARE_POOL_ALLOCATOR();
 };
 
-PDECLARE_BASEARRAY(PHashTableInfo, PHashTableElement *)
-#ifdef DOC_PLUS_PLUS
+class PHashTableInfo : public PBaseArray<PHashTableElement *>
 {
-#endif
+    typedef PBaseArray<PHashTableElement *> ParentClass;
+    PCLASSINFO(PCharArray, ParentClass);
   public:
+    PHashTableInfo(PINDEX initialSize = 0)
+      : ParentClass(initialSize) { }
+    PHashTableInfo(PHashTableElement * const * buffer, PINDEX length, PBoolean dynamic = true)
+      : ParentClass(buffer, length, dynamic) { }
+    virtual PObject * Clone() const { return PNEW PHashTableInfo(*this, GetSize()); }
     virtual ~PHashTableInfo() { Destruct(); }
     virtual void DestroyContents();
 
     PINDEX AppendElement(PObject * key, PObject * data);
     PObject * RemoveElement(const PObject & key);
-    PBoolean SetLastElementAt(PINDEX index, PHashTableElement * & lastElement);
+    PHashTableElement * GetElementAt(PINDEX index);
     PHashTableElement * GetElementAt(const PObject & key);
     PINDEX GetElementsIndex(const PObject*obj,PBoolean byVal,PBoolean keys) const;
+    PHashTableElement * NextElement(PHashTableElement * element) const;
+    PHashTableElement * PrevElement(PHashTableElement * element) const;
 
     PBoolean deleteKeys;
 
@@ -241,8 +268,10 @@ class PHashTable : public PCollection
        The ordinal position in the hash table is determined by the hash values
        of the keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        This function is primarily used by the descendent template classes, or
        macro, with the appropriate type conversion.
@@ -259,8 +288,10 @@ class PHashTable : public PCollection
        The ordinal position in the hash table is determined by the hash values
        of the keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        This function is primarily used by the descendent template classes, or
        macro, with the appropriate type conversion.
@@ -363,6 +394,11 @@ class PAbstractSet : public PHashTable
     /**Remove an object at the specified index. If the <code>AllowDeleteObjects</code>
        option is set then the object is also deleted.
 
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
+
        @return
        pointer to the object being removed, or NULL if it was deleted.
      */
@@ -372,6 +408,11 @@ class PAbstractSet : public PHashTable
 
     /**This function is the same as PHashTable::AbstractGetKeyAt().
 
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
+
        @return
        Always NULL.
      */
@@ -467,7 +508,7 @@ template <class T> class PSet : public PAbstractSet
        This is different from the default on most collection classes.
      */
     inline PSet(PBoolean initialDeleteObjects = false)
-      : PAbstractSet() { AllowDeleteObjects(initialDeleteObjects); }
+      : PAbstractSet() { this->AllowDeleteObjects(initialDeleteObjects); }
   //@}
 
   /**@name Overrides from class PObject */
@@ -492,7 +533,7 @@ template <class T> class PSet : public PAbstractSet
      */
     void Include(
       const T * obj   // New object to include in the set.
-    ) { Append((PObject *)obj); }
+    ) { this->Append((PObject *)obj); }
 
     /**Include the specified objects value into the set. If the objects value
        is already in the set then the object is \b not included.
@@ -503,7 +544,7 @@ template <class T> class PSet : public PAbstractSet
      */
     PSet & operator+=(
       const T & obj   // New object to include in the set.
-    ) { Append(obj.Clone()); return *this; }
+    ) { this->Append(obj.Clone()); return *this; }
 
     /**Remove the object from the set. If the <code>AllowDeleteObjects</code> option is set
        then the object is also deleted.
@@ -514,7 +555,7 @@ template <class T> class PSet : public PAbstractSet
      */
     void Exclude(
       const T * obj   // New object to exclude in the set.
-    ) { Remove(obj); }
+    ) { this->Remove(obj); }
 
     /**Remove the objects value from the set. If the <code>AllowDeleteObjects</code>
        option is set then the object is also deleted.
@@ -525,7 +566,7 @@ template <class T> class PSet : public PAbstractSet
      */
     PSet & operator-=(
       const T & obj   // New object to exclude in the set.
-    ) { RemoveAt(GetValuesIndex(obj)); return *this; }
+    ) { this->erase(this->find(obj)); return *this; }
 
     /**Determine if the value of the object is contained in the set. The
        object values are compared, not the pointers.  So the objects in the
@@ -537,7 +578,7 @@ template <class T> class PSet : public PAbstractSet
      */
     PBoolean Contains(
       const T & key  ///< Key to look for in the set.
-    ) const { return AbstractContains(key); }
+    ) const { return this->AbstractContains(key); }
 
     /**Determine if the value of the object is contained in the set. The
        object values are compared, not the pointers.  So the objects in the
@@ -549,15 +590,17 @@ template <class T> class PSet : public PAbstractSet
      */
     PBoolean operator[](
       const T & key  ///< Key to look for in the set.
-    ) const { return AbstractContains(key); }
+    ) const { return this->AbstractContains(key); }
 
     /**Get the key in the set at the ordinal index position.
 
        The ordinal position in the set is determined by the hash values of the
        keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        @return
        reference to key at the index position.
@@ -565,14 +608,96 @@ template <class T> class PSet : public PAbstractSet
     virtual const T & GetKeyAt(
       PINDEX index    ///< Index of value to get.
     ) const
-      { return (const T &)AbstractGetKeyAt(index); }
+      { return dynamic_cast<const T &>(this->AbstractGetKeyAt(index)); }
   //@}
 
+  /**@name Iterators */
+  //@{
+    class iterator;
+    class const_iterator;
+    class iterator_base : public std::iterator<std::forward_iterator_tag, T> {
+      protected:
+        iterator_base()
+          : table(NULL)
+          , element(NULL)
+          { }
+        iterator_base(PHashTableInfo * t)
+          : table(t)
+          , element(t->GetElementAt((PINDEX)0))
+          { }
+        iterator_base(PHashTableInfo * t, const T & k)
+          : table(t)
+          , element(t->GetElementAt(k))
+          { }
+
+        PHashTableInfo    * table;
+        PHashTableElement * element;
+
+        void Next() { this->element = PAssertNULL(this->table)->NextElement(this->element); }
+        void Prev() { this->element = PAssertNULL(this->table)->PrevElement(this->element); }
+
+        T * Ptr() const { return dynamic_cast<T *>(PAssertNULL(this->element)->key); }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->element == it.element; }
+        bool operator!=(const iterator_base & it) const { return this->element != it.element; }
+    };
+
+    class iterator : public iterator_base  {
+      protected:
+        iterator(PHashTableInfo * t) : iterator_base(t) { }
+        iterator(PHashTableInfo * t, const T & k) : iterator_base(t, k) { }
+
+      public:
+        iterator() { }
+
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
+
+        T * operator->() const { return  this->Ptr(); }
+        T & operator* () const { return *this->Ptr(); }
+
+      friend class PSet<T>;
+    };
+
+    iterator begin() { return iterator(this->hashTable); }
+    iterator end()   { return iterator(); }
+    iterator find(const T & k) { return iterator(this->hashTable, k); }
+
+
+    class const_iterator : public iterator_base {
+      protected:
+        const_iterator(PHashTableInfo * t) : iterator_base(t) { }
+        const_iterator(PHashTableInfo * t, const T & k) : iterator_base(t, k) { }
+
+      public:
+        const_iterator() { }
+
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
+
+        const T * operator->() const { return  this->Ptr(); }
+        const T & operator* () const { return *this->Ptr(); }
+
+      friend class PSet<T>;
+    };
+
+    const_iterator begin() const { return const_iterator(this->hashTable); }
+    const_iterator end()   const { return const_iterator(); }
+    const_iterator find(const T & k) const { return const_iterator(this->hashTable, k); }
+
+    void erase(const iterator & it) { this->Remove(&*it); }
+    void erase(const const_iterator & it) { this->Remove(&*it); }
+  //@}
 
   protected:
     PSet(int dummy, const PSet * c)
       : PAbstractSet(dummy, c)
-      { reference->deleteObjects = c->reference->deleteObjects; }
+      { this->reference->deleteObjects = c->reference->deleteObjects; }
 };
 
 
@@ -602,21 +727,21 @@ template <class T> class PSet : public PAbstractSet
    See the <code>PSet</code> and <code>PAbstractSet</code> classes for more information.
  */
 #define PDECLARE_SET(cls, T, initDelObj) \
-  PSET(cls##_PTemplate, T); \
-  PDECLARE_CLASS(cls, cls##_PTemplate) \
+  class cls : public PSet<T> { \
+    typedef PSet<T> BaseClass; PCLASSINFO(cls, BaseClass) \
   protected: \
     cls(int dummy, const cls * c) \
-      : cls##_PTemplate(dummy, c) { } \
+      : BaseClass(dummy, c) { } \
   public: \
     cls(PBoolean initialDeleteObjects = initDelObj) \
-      : cls##_PTemplate(initialDeleteObjects) { } \
+      : BaseClass(initialDeleteObjects) { } \
     virtual PObject * Clone() const \
       { return PNEW cls(0, this); } \
 
 
-
-PSET(POrdinalSet, POrdinalKey);
-
+/// A set of ordinal integers
+PDECLARE_SET(POrdinalSet, POrdinalKey, true)
+};
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -750,7 +875,7 @@ class PAbstractDictionary : public PHashTable
        @return
        true if the new object could be placed into the dictionary.
      */
-    virtual PBoolean SetDataAt(
+    P_DEPRECATED virtual PBoolean SetDataAt(
       PINDEX index,   ///< Ordinal index in the dictionary.
       PObject * obj   ///< New object to put into the dictionary.
     );
@@ -766,7 +891,7 @@ class PAbstractDictionary : public PHashTable
        @return
        true if the object was successfully added.
      */
-    virtual PBoolean AbstractSetAt(
+    virtual PObject * AbstractSetAt(
       const PObject & key,  ///< Key for position in dictionary to add object.
       PObject * obj         ///< New object to put into the dictionary.
     );
@@ -843,6 +968,10 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
   PCLASSINFO(PDictionary, PAbstractDictionary);
 
   public:
+    typedef K key_type;
+    typedef D data_type;
+    typedef PDictionary<K, D> dict_type;
+
   /**@name Construction */
   //@{
     /**Create a new, empty, dictionary.
@@ -877,10 +1006,12 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
        @return
        reference to the object indexed by the key.
      */
+    const D & operator[](
+      const K & key   ///< Key to look for in the dictionary.
+    ) const { return dynamic_cast<const D &>(this->GetRefAt(key)); }
     D & operator[](
       const K & key   ///< Key to look for in the dictionary.
-    ) const
-      { return (D &)GetRefAt(key); }
+    ) { return dynamic_cast<D &>(this->GetRefAt(key)); }
 
     /**Determine if the value of the object is contained in the hash table. The
        object values are compared, not the pointers.  So the objects in the
@@ -892,7 +1023,7 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
      */
     PBoolean Contains(
       const K & key   ///< Key to look for in the dictionary.
-    ) const { return AbstractContains(key); }
+    ) const { return this->AbstractContains(key); }
 
     /**Remove an object at the specified \p key. The returned pointer is then
        removed using the <code>SetAt()</code> function to set that key value to
@@ -901,16 +1032,11 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
 
        @return
        pointer to the object being removed, or NULL if the key was not
-       present in the dictionary. If the dictionary is set to delete objects
-       upon removal, the value -1 is returned if the key existed prior to removal
-       rather than returning an illegal pointer
+       present in the dictionary or the object was deleted automatically.
      */
     virtual D * RemoveAt(
       const K & key   ///< Key for position in dictionary to get object.
-    ) {
-        D * obj = GetAt(key); AbstractSetAt(key, NULL);
-        return reference->deleteObjects ? (obj ? (D *)-1 : NULL) : obj;
-      }
+    ) { return dynamic_cast<D *>(this->AbstractSetAt(key, NULL)); }
 
     /**Add a new object to the collection. If the objects value is already in
        the dictionary then the object is overrides the previous value. If the
@@ -926,7 +1052,7 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
     virtual PBoolean SetAt(
       const K & key,  // Key for position in dictionary to add object.
       D * obj         // New object to put into the dictionary.
-    ) { return AbstractSetAt(key, obj); }
+    ) { return this->AbstractSetAt(key, obj) != NULL; }
 
     /**Get the object at the specified key position. If the key was not in the
        collection then NULL is returned.
@@ -936,15 +1062,17 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
      */
     virtual D * GetAt(
       const K & key   // Key for position in dictionary to get object.
-    ) const { return (D *)AbstractGetAt(key); }
+    ) const { return dynamic_cast<D *>(this->AbstractGetAt(key)); }
 
     /**Get the key in the dictionary at the ordinal index position.
 
        The ordinal position in the dictionary is determined by the hash values
        of the keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        @return
        reference to key at the index position.
@@ -952,15 +1080,17 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
     const K & GetKeyAt(
       PINDEX index  ///< Ordinal position in dictionary for key.
     ) const
-      { return (const K &)AbstractGetKeyAt(index); }
+      { return dynamic_cast<const K &>(this->AbstractGetKeyAt(index)); }
 
     /**Get the data in the dictionary at the ordinal index position.
 
        The ordinal position in the dictionary is determined by the hash values
        of the keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        @return
        reference to data at the index position.
@@ -968,19 +1098,134 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
     D & GetDataAt(
       PINDEX index  ///< Ordinal position in dictionary for data.
     ) const
-      { return (D &)AbstractGetDataAt(index); }
+      { return dynamic_cast<D &>(this->AbstractGetDataAt(index)); }
 
     /**Get an array containing all the keys for the dictionary.
       */
     PArray<K> GetKeys() const
     {
       PArray<K> keys;
-      AbstractGetKeys(keys);
+      this->AbstractGetKeys(keys);
       return keys;
     }
   //@}
 
-    typedef std::pair<K, D *> value_type;
+  /**@name Iterators */
+  //@{
+    class iterator;
+    class const_iterator;
+    class iterator_base {
+      protected:
+        K * m_internal_first;  // Must be first two members
+        D * m_internal_second;
+
+        PHashTableInfo    * m_table;
+        PHashTableElement * m_element;
+
+        iterator_base()
+          : m_internal_first(NULL)
+          , m_internal_second(NULL)
+          , m_table(NULL)
+          , m_element(NULL)
+        {
+        }
+
+        iterator_base(const dict_type * dict)
+          : m_table(dict->hashTable)
+        {
+          this->SetElement(this->m_table->GetElementAt((PINDEX)0));
+        }
+        
+        iterator_base(const dict_type * dict, const K & key)
+          : m_table(dict->hashTable)
+        {
+          this->SetElement(this->m_table->GetElementAt(key));
+        }
+
+        void SetElement(PHashTableElement * element)
+        {
+          this->m_element = element;
+          if (element != NULL) {
+            this->m_internal_first  = dynamic_cast<K *>(element->key);
+            this->m_internal_second = dynamic_cast<D *>(element->data);
+          }
+          else {
+            this->m_internal_first = NULL;
+            this->m_internal_second = NULL;
+          }
+        }
+
+        void Next() { this->SetElement(PAssertNULL(this->m_table)->NextElement(this->m_element)); }
+        void Prev() { this->SetElement(PAssertNULL(this->m_table)->PrevElement(this->m_element)); }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->m_element == it.m_element; }
+        bool operator!=(const iterator_base & it) const { return this->m_element != it.m_element; }
+    };
+
+    template<class CK, class CD>
+    class iterator_pair {
+      public:
+        CK & first;
+        CD & second;
+
+      private:
+        iterator_pair() : first(reinterpret_cast<CK&>(0)), second(reinterpret_cast<CD&>(0)) { }
+    };
+
+    class iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair<K,D> > {
+      protected:
+        iterator(dict_type * dict) : iterator_base(dict) { }
+        iterator(dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        iterator() { }
+
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
+
+        typedef iterator_pair<K,D> pair;
+        const pair * operator->() const { return  reinterpret_cast<const pair *>(this); }
+        const pair & operator* () const { return *reinterpret_cast<const pair *>(this); }
+
+      friend class PDictionary<K, D>;
+    };
+
+    iterator begin() { return iterator(this); }
+    iterator end()   { return iterator(); }
+    iterator find(const K & key) { return iterator(this, key); }
+
+
+    class const_iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair<const K,const D> > {
+      protected:
+        const_iterator(const dict_type * dict) : iterator_base(dict) { }
+        const_iterator(const dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        const_iterator() { }
+        const_iterator(iterator it) : iterator_base(it) { }
+
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
+
+        typedef iterator_pair<const K, const D> pair;
+        const pair * operator->() const { return  reinterpret_cast<const pair *>(this); }
+        const pair & operator* () const { return *reinterpret_cast<const pair *>(this); }
+
+      friend class PDictionary<K, D>;
+    };
+
+    const_iterator begin() const { return const_iterator(this); }
+    const_iterator end()   const { return const_iterator(); }
+    const_iterator find(const K & k) const { return const_iterator(this, k); }
+
+    void erase(const       iterator & it) { this->AbstractSetAt(*it.m_element->key, NULL); }
+    void erase(const const_iterator & it) { this->AbstractSetAt(*it.m_element->key, NULL); }
+  //@}
 
   protected:
     PDictionary(int dummy, const PDictionary * c)
@@ -1035,9 +1280,10 @@ template <class K, class D> class PDictionary : public PAbstractDictionary
    Note that if templates are not used the <code>PDECLARE_ORDINAL_DICTIONARY</code>
    macro will simulate the template instantiation.
  */
-template <class K> class POrdinalDictionary : public PAbstractDictionary
+template <class K> class POrdinalDictionary : public PDictionary<K, POrdinalKey>
 {
-  PCLASSINFO(POrdinalDictionary, PAbstractDictionary);
+  typedef PDictionary<K, POrdinalKey> ParentClass;
+  PCLASSINFO(POrdinalDictionary, ParentClass);
 
   public:
   /**@name Construction */
@@ -1049,7 +1295,7 @@ template <class K> class POrdinalDictionary : public PAbstractDictionary
        destroyed.
      */
     POrdinalDictionary()
-      : PAbstractDictionary() { }
+      : ParentClass() { }
   //@}
 
   /**@name Overrides from class PObject */
@@ -1077,42 +1323,25 @@ template <class K> class POrdinalDictionary : public PAbstractDictionary
     PINDEX operator[](
       const K & key   // Key to look for in the dictionary.
     ) const
-      { return (POrdinalKey &)GetRefAt(key); }
-
-    /**Determine if the value of the object is contained in the hash table. The
-       object values are compared, not the pointers.  So the objects in the
-       collection must correctly implement the <code>PObject::Compare()</code>
-       function. The hash table is used to locate the entry.
-
-       @return
-       true if the object value is in the dictionary.
-     */
-    PBoolean Contains(
-      const K & key   ///< Key to look for in the dictionary.
-    ) const { return AbstractContains(key); }
-
-    virtual POrdinalKey * GetAt(
-      const K & key   ///< Key for position in dictionary to get object.
-    ) const { return (POrdinalKey *)AbstractGetAt(key); }
-    /* Get the object at the specified key position. If the key was not in the
-       collection then NULL is returned.
-
-       @return
-       pointer to object at the specified key.
-     */
+      { return dynamic_cast<POrdinalKey &>(this->GetRefAt(key)); }
 
     /**Set the data at the specified ordinal index position in the dictionary.
 
        The ordinal position in the dictionary is determined by the hash values
        of the keys and the order of insertion.
 
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
+
        @return
        true if the new object could be placed into the dictionary.
      */
     virtual PBoolean SetDataAt(
       PINDEX index,   ///< Ordinal index in the dictionary.
       PINDEX ordinal  ///< New ordinal value to put into the dictionary.
-      ) { return PAbstractDictionary::SetDataAt(index, PNEW POrdinalKey(ordinal)); }
+    ) { return this->AbstractSetAt(this->AbstractGetKeyAt(index), PNEW POrdinalKey(ordinal)) != NULL; }
 
     /**Add a new object to the collection. If the objects value is already in
        the dictionary then the object is overrides the previous value. If the
@@ -1128,27 +1357,17 @@ template <class K> class POrdinalDictionary : public PAbstractDictionary
     virtual PBoolean SetAt(
       const K & key,  ///< Key for position in dictionary to add object.
       PINDEX ordinal  ///< New ordinal value to put into the dictionary.
-    ) { return AbstractSetAt(key, PNEW POrdinalKey(ordinal)); }
-
-    /**Remove an object at the specified key. The returned pointer is then
-       removed using the <code>SetAt()</code> function to set that key value to
-       NULL. If the <code>AllowDeleteObjects</code> option is set then the
-       object is also deleted.
-
-       @return
-       pointer to the object being removed, or NULL if it was deleted.
-     */
-    virtual PINDEX RemoveAt(
-      const K & key   ///< Key for position in dictionary to get object.
-    ) { PINDEX ord = *GetAt(key); AbstractSetAt(key, NULL); return ord; }
+    ) { return this->AbstractSetAt(key, PNEW POrdinalKey(ordinal)) != NULL; }
 
     /**Get the key in the dictionary at the ordinal index position.
 
        The ordinal position in the dictionary is determined by the hash values
        of the keys and the order of insertion.
 
-       The last key/data pair is remembered by the class so that subseqent
-       access is very fast.
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
 
        @return
        reference to key at the index position.
@@ -1156,7 +1375,7 @@ template <class K> class POrdinalDictionary : public PAbstractDictionary
     const K & GetKeyAt(
       PINDEX index  ///< Ordinal position in dictionary for key.
     ) const
-      { return (const K &)AbstractGetKeyAt(index); }
+      { return dynamic_cast<const K &>(this->AbstractGetKeyAt(index)); }
 
     /**Get the data in the dictionary at the ordinal index position.
 
@@ -1166,18 +1385,23 @@ template <class K> class POrdinalDictionary : public PAbstractDictionary
        The last key/data pair is remembered by the class so that subseqent
        access is very fast.
 
+       WARNING: this code requires a linear search for every operation
+                and will be VERY slow for large data sets. This function
+                is kept for backwards compatibilty only. New code should
+                use the iterator based access methods
+
        @return
        reference to data at the index position.
      */
     PINDEX GetDataAt(
       PINDEX index  ///< Ordinal position in dictionary for data.
     ) const
-      { return (POrdinalKey &)AbstractGetDataAt(index); }
+      { return dynamic_cast<POrdinalKey &>(this->AbstractGetDataAt(index)); }
   //@}
 
   protected:
     POrdinalDictionary(int dummy, const POrdinalDictionary * c)
-      : PAbstractDictionary(dummy, c) { }
+      : ParentClass(dummy, c) { }
 };
 
 
