@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27062 $
+ * $Revision: 29742 $
  * $Author: rjongbloed $
- * $Date: 2012-02-27 00:17:08 -0600 (Mon, 27 Feb 2012) $
+ * $Date: 2013-05-20 13:27:45 +1000 (Mon, 20 May 2013) $
  */
 
 #ifndef PTLIB_IPSOCKET_H
@@ -40,18 +40,6 @@
 
 #include <ptlib/socket.h>
 
-#if P_QOS
-#ifdef _WIN32
-#ifdef P_KNOCKOUT_WINSOCK2
-   #include "IPExport.h"
-#endif // KNOCKOUT_WINSOCK2
-#endif // _WIN32
-#endif // P_QOS
-
-
-class PIPSocketAddressAndPort;
-
-
 
 /**This class describes a type of socket that will communicate using the
    Internet Protocol.
@@ -73,6 +61,7 @@ class PIPSocket : public PSocket
     /**A class describing an IP address.
      */
     class Address : public PObject {
+        PCLASSINFO_WITH_CLONE(Address, PObject);
       public:
 
         /**@name Address constructors */
@@ -83,10 +72,10 @@ class PIPSocket : public PSocket
         /**Create an IP address from string notation,
            eg dot notation x.x.x.x. for IPv4, or colon notation x:x:x::xxx for IPv6.
           */
-        Address(const PString & dotNotation);
+        explicit Address(const PString & dotNotation);
 
         /// Create an IPv4 or IPv6 address from 4 or 16 byte values.
-        Address(PINDEX len, const BYTE * bytes);
+        Address(PINDEX len, const BYTE * bytes, int scope = 0);
 
         /// Create an IP address from four byte values.
         Address(BYTE b1, BYTE b2, BYTE b3, BYTE b4);
@@ -100,6 +89,7 @@ class PIPSocket : public PSocket
 #if P_HAS_IPV6
         /// Create an IPv6 address from an in_addr structure.
         Address(const in6_addr & addr);
+        Address(const in6_addr & addr, int scope);
 #endif
 
         /// Create an IP (v4 or v6) address from a sockaddr (sockaddr_in,
@@ -116,7 +106,7 @@ class PIPSocket : public PSocket
 
 #if P_HAS_IPV6
         /// Copy an address from another IPv6 address.
-        Address & operator=(const in6_addr & addr);
+        Address & AssignIPV6(const in6_addr & addr, int scope);
 #endif
 
         /// Copy an address from a string.
@@ -131,8 +121,11 @@ class PIPSocket : public PSocket
         bool operator==(const Address & addr) const { return Compare(addr) == EqualTo; }
         bool operator!=(const Address & addr) const { return Compare(addr) != EqualTo; }
 #if P_HAS_IPV6
-        bool operator==(in6_addr & addr) const;
-        bool operator!=(in6_addr & addr) const { return !operator==(addr); }
+        bool operator ==(in6_addr & addr) const;
+        bool operator !=(in6_addr & addr) const { return !operator==(addr); }
+
+        bool EqualIPV6(in6_addr & addr, int scope) const;
+        bool NotEqualIPV6(in6_addr & addr, int scope) const { return !EqualIPV6(addr, scope); }
 #endif
         bool operator==(in_addr & addr) const;
         bool operator!=(in_addr & addr) const { return !operator==(addr); }
@@ -167,7 +160,8 @@ class PIPSocket : public PSocket
 
         /// Format an address as a string.
         PString AsString(
-          bool bracketIPv6 = false ///< An IPv6 address is enclosed in []'s
+          bool bracketIPv6 = false,  ///< An IPv6 address is enclosed in []'s
+          bool excludeScope = false  ///< An IPv6 address includes %xxx for scope ID
         ) const;
 
         /// Convert string to IP address. Returns true if was a valid address.
@@ -182,8 +176,10 @@ class PIPSocket : public PSocket
         operator in_addr() const;
 
 #if P_HAS_IPV6
-        /// Return IPv4 address in network order.
+        /// Return IPv6 address in network order.
         operator in6_addr() const;
+
+        int GetIPV6Scope() const { return m_scope6; }
 #endif
 
         /// Return IPv4 address in network order.
@@ -208,23 +204,24 @@ class PIPSocket : public PSocket
         PINDEX GetSize() const;
 
         /// Get the pointer to IP address data.
-        const char * GetPointer() const { return (const char *)&v; }
+        const char * GetPointer() const { return (const char *)&m_v; }
 
         /// Get the version of the IP address being used.
-        unsigned GetVersion() const { return version; }
+        unsigned GetVersion() const { return m_version; }
 
+        /// Check for illegal address
+        bool IsValid() const { return m_version == 4 || m_version == 6; }
         /// Check address 0.0.0.0 or ::.
-        PBoolean IsValid() const;
-        PBoolean IsAny() const;
+        bool IsAny() const;
 
         /// Check address 127.0.0.1 or ::1.
-        PBoolean IsLoopback() const;
+        bool IsLoopback() const;
 
         /// Check for Broadcast address 255.255.255.255.
-        PBoolean IsBroadcast() const;
+        bool IsBroadcast() const;
 
         /// Check if address is multicast group
-        PBoolean IsMulticast() const;
+        bool IsMulticast() const;
 
         /** Check if the remote address is a private address.
             For IPV4 this is specified RFC 1918 as the following ranges:
@@ -234,37 +231,127 @@ class PIPSocket : public PSocket
 
             For IPV6 this is specified as any address having "1111 1110 1" for the first nine bits.
         */
-        PBoolean IsRFC1918() const ;
+        bool IsRFC1918() const ;
 
 #if P_HAS_IPV6
-        /// Check for v4 mapped i nv6 address ::ffff:a.b.c.d.
-        PBoolean IsV4Mapped() const;
+        /// Check for v4 mapped in v6 address ::ffff:a.b.c.d.
+        bool IsV4Mapped() const;
 
-		/// Check for link-local address fe80::/10
-        PBoolean IsLinkLocal() const;
+            /// Check for link-local address 
+        bool IsLinkLocal() const;
+
+        //// Check for site-local address
+        bool IsSiteLocal() const;
 #endif
 
-        static const Address & GetLoopback(int version = 4);
-        static const Address & GetAny(int version = 4);
-        static const Address GetBroadcast(int version = 4);
+        static const Address & GetLoopback(unsigned version = 4);
+        static const Address & GetAny(unsigned version = 4);
+        static const Address GetBroadcast(unsigned version = 4);
 
       protected:
         /// Runtime test of IP addresse type.
         union {
-          in_addr four;
+          in_addr m_four;
 #if P_HAS_IPV6
-          in6_addr six;
+          in6_addr m_six;
 #endif
-        } v;
-        unsigned version;
+        } m_v;
+        unsigned m_version;
+        int      m_scope6;
 
-      /// Output IPv6 & IPv4 address as a string to the specified string.
+      /** Output IPv6 & IPv4 address as a string to the specified string.
+          If the stream flag <b>hex</b> is set, then an IPv6 address is
+          surrounded by square brackets [].
+          If the stream flag <b>fixed</b> is set, the an IPv6 address will
+          have the scope ID (%xxx) suppressed.
+        */
       friend ostream & operator<<(ostream & s, const Address & a);
 
       /// Input IPv4 (not IPv6 yet!) address as a string from the specified string.
       friend istream & operator>>(istream & s, Address & a);
     };
 
+    /**A class describing an IP address and port number combination.
+     */
+    class AddressAndPort
+    {
+      public:
+        AddressAndPort(
+          char separator = ':'
+        );
+        AddressAndPort(
+          WORD defaultPort,
+          char separator = ':'
+        );
+        AddressAndPort(
+          const PString & str,
+          WORD defaultPort = 0,
+          char separator = ':',
+          const char * proto = NULL
+        );
+        AddressAndPort(
+          const PIPSocket::Address & addr,
+          WORD defaultPort = 0,
+          char separator = ':'
+        );
+        AddressAndPort(
+          struct sockaddr *ai_addr,
+          const int ai_addrlen
+        );
+
+        bool Parse(
+          const PString & str,
+          WORD defaultPort = 0,
+          char separator = ':',
+          const char * proto = NULL
+        );
+
+        PString AsString(char separator = 0) const;
+
+        const PIPSocket::Address & GetAddress() const { return m_address; }
+
+        void SetAddress(
+          const PIPSocket::Address & addr,
+          WORD port = 0
+        );
+
+        WORD GetPort() const { return m_port; }
+
+        void SetPort(
+          WORD port
+        ) { m_port = port; }
+
+        bool IsValid() const
+        {
+          return m_address.IsValid() && m_port != 0;
+        }
+
+        bool operator==(const AddressAndPort & obj) const
+        {
+          return m_port == obj.m_port && m_address == obj.m_address;
+        }
+
+        bool operator!=(const AddressAndPort & obj) const
+        {
+          return m_port != obj.m_port || m_address != obj.m_address;
+        }
+
+        friend ostream & operator<<(ostream & strm, const AddressAndPort & ap)
+        {
+          return strm << ap.AsString();
+        }
+
+        bool MatchWildcard(
+          const AddressAndPort & wild
+        ) const;
+
+      protected:
+        PIPSocket::Address m_address;
+        WORD               m_port;
+        char               m_separator;
+    };
+
+
     //**@name Overrides from class PChannel */
     //@{
     /**Get the platform and I/O channel type name of the channel. For an IP
@@ -288,11 +375,25 @@ class PIPSocket : public PSocket
 #if P_HAS_IPV6
     static void SetDefaultIpAddressFamilyV6(); // PF_INET6
     static PBoolean IsIpAddressFamilyV6Supported();
-
-	static void SetDefaultV6ScopeId(int scopeId); // local-link adresses require one
-    static int GetDefaultV6ScopeId(); 
 #endif
-    static PIPSocket::Address GetDefaultIpAny();
+    static const PIPSocket::Address & GetDefaultIpAny();
+    static const PIPSocket::Address & GetInvalidAddress();
+
+    /**Set flag for suppress getting canonical name when doing lookup via
+       hostname.
+
+       Some badly configured DNS servers can cause long delays when this
+       feature is used.
+      */
+    static void SetSuppressCanonicalName(bool suppress);
+
+    /**Get flag for suppress getting canonical name when doing lookup via
+       hostname.
+
+       Some badly configured DNS servers can cause long delays when this
+       feature is used.
+      */
+    static bool GetSuppressCanonicalName();
 
     /**Open an IPv4 or IPv6 socket
      */
@@ -353,13 +454,14 @@ class PIPSocket : public PSocket
       unsigned queueSize = 5,  ///< Number of pending accepts that may be queued.
       WORD port = 0,           ///< Port number to use for the connection.
       Reusability reuse = AddressIsExclusive ///< Can/Cant listen more than once.
-    );
+    ) { return InternalListen(GetDefaultIpAny(), queueSize, port, reuse); }
+
     virtual PBoolean Listen(
       const Address & bind,     ///< Local interface address to bind to.
       unsigned queueSize = 5,   ///< Number of pending accepts that may be queued.
       WORD port = 0,            ///< Port number to use for the connection.
       Reusability reuse = AddressIsExclusive ///< Can/Can't listen more than once.
-    );
+    ) { return InternalListen(bind, queueSize, port, reuse); }
     //@}
 
     /**@name New functions for class */
@@ -435,17 +537,18 @@ class PIPSocket : public PSocket
        @return
        false (or empty string) if the IP number was not available.
      */
-    virtual PString GetLocalAddress();
-    virtual PBoolean GetLocalAddress(
+    PString GetLocalAddress();
+    bool GetLocalAddress(
       Address & addr    ///< Variable to receive hosts IP address.
     );
-    virtual PBoolean GetLocalAddress(
-      PIPSocketAddressAndPort & addr    ///< Variable to receive hosts IP address and port.
-    );
-    virtual PBoolean GetLocalAddress(
+    bool GetLocalAddress(
       Address & addr,    ///< Variable to receive peer hosts IP address.
       WORD & port        ///< Variable to receive peer hosts port number.
     );
+    bool GetLocalAddress(
+      AddressAndPort & addr    ///< Variable to receive hosts IP address and port.
+    )
+    { return InternalGetLocalAddress(addr); }
 
     /**Get the Internet Protocol address for the peer host and port the
        socket is connected to.
@@ -453,17 +556,18 @@ class PIPSocket : public PSocket
        @return
        false (or empty string) if the IP number was not available.
      */
-    virtual PString GetPeerAddress();
-    virtual PBoolean GetPeerAddress(
+    PString GetPeerAddress();
+    bool GetPeerAddress(
       Address & addr    ///< Variable to receive hosts IP address.
     );
-    virtual PBoolean GetPeerAddress(
-      PIPSocketAddressAndPort & addr    ///< Variable to receive hosts IP address and port.
-    );
-    virtual PBoolean GetPeerAddress(
+    bool GetPeerAddress(
       Address & addr,    ///< Variable to receive peer hosts IP address.
       WORD & port        ///< Variable to receive peer hosts port number.
     );
+    bool GetPeerAddress(
+      AddressAndPort & addr    ///< Variable to receive hosts IP address and port.
+    )
+    { return InternalGetPeerAddress(addr); }
 
     /**Get the host name for the local host.
 
@@ -483,78 +587,6 @@ class PIPSocket : public PSocket
      */
     static void ClearNameCache();
 
-    /**Get the IP address that is being used as the gateway, that is, the
-       computer that packets on the default route will be sent.
-
-       The string returned may be used in the Connect() function to open that
-       interface.
-
-       Note that the driver does not need to be open for this function to work.
-
-       @return
-       true if there was a gateway.
-     */
-    static PBoolean GetGatewayAddress(
-      Address & addr,     ///< Variable to receive the IP address.
-	  int version = 4     ///< IP version number
-    );
-
-    /**Get the name for the interface that is being used as the gateway,
-       that is, the interface that packets on the default route will be sent.
-
-       The string returned may be used in the Connect() function to open that
-       interface.
-
-       Note that the driver does not need to be open for this function to work.
-
-       @return
-       String name of the gateway device, or empty string if there is none.
-     */
-    static PString GetGatewayInterface(int version = 4);
-
-    /**Get the interface address that will be used to reach the specified
-       remote address. Uses longest prefix match when multiple matching interfaces
-       are found.
-
-       @return
-       Network interface address.
-      */
-    static PIPSocket::Address GetRouteInterfaceAddress(PIPSocket::Address remoteAddress);
-
-#ifdef _WIN32
-    /**Get the IP address for the interface that is being used as the gateway,
-       that is, the interface that packets on the default route will be sent.
-
-       This Function can be used to Bind the Listener to only the default Packet
-       route in DHCP Environs.
-
-       Note that the driver does not need to be open for this function to work.
-
-       @return
-       The Local Interface IP Address for Gatway Access.
-     */
-    static PIPSocket::Address GetGatewayInterfaceAddress(int version = 4);
-
-    /**Retrieve the Local IP Address for which packets would have be routed to the to reach the remote Address.
-       @return Local Address.
-    */
-    static PIPSocket::Address GetRouteAddress(PIPSocket::Address RemoteAddress);
-
-    /**IP Address to a Numerical Representation.
-     */
-    static unsigned AsNumeric(Address addr);
-
-    /**Check if packets on Interface Address can reach the remote IP Address.
-     */
-    static PBoolean IsAddressReachable(PIPSocket::Address LocalIP,
-                                   PIPSocket::Address LocalMask,
-                                   PIPSocket::Address RemoteIP);
-
-    /**Get the Interface Name for a given local Interface Address.
-     */
-    static PString GetInterface(PIPSocket::Address addr);
-    //@}
- #endif
     /**Describe a route table entry.
      */
     class RouteEntry : public PObject
@@ -579,6 +611,9 @@ class PIPSocket : public PSocket
         /// Get the network metric associated with the route table entry.
         long GetMetric() const { return metric; }
 
+        ///< Print the route table entry
+        void PrintOn(ostream & strm) const;
+
       protected:
         Address network;
         Address net_mask;
@@ -598,7 +633,7 @@ class PIPSocket : public PSocket
      */
     static PBoolean GetRouteTable(
       RouteTable & table      ///< Route table
-	);
+    );
 
     /// Class for detector of Route Table changes
     class RouteTableDetector
@@ -684,47 +719,135 @@ class PIPSocket : public PSocket
       PBoolean includeDown = false     ///< Include interfaces that are down
     );
 
-    /**Get the address of an interface that corresponds to a real network.
+    /**Get the interface name for the specified local IP address.
+      */
+    static PString GetInterface(
+      const Address & addr    ///< IP address of interface
+    );
+
+    /**Get the interface name for the specified local IP address.
+      */
+    static Address GetInterfaceAddress(
+      const PString & ifName,   ///< Name of interface
+      unsigned version = 4      ///< IP version number
+    );
+
+    /**Get the address of the first interface to the "Internet".
+       This searches the interfaces for one that has a "public" IP address and
+       thus would be access to the Internet.
+
+       @return false if only loopback/private interfaces could be found.
+     */
+    static Address GetNetworkInterface(
+      unsigned version = 4  ///< IP version number
+    );
+
+    /**Get the IP address that is being used as the gateway, that is, the
+       computer that packets on the default route will be sent.
+
+       The string returned may be used in the Connect() function to open that
+       interface.
+
+       Note that the driver does not need to be open for this function to work.
+
        @return
-       false if only loopback interfaces could be found, else true.
+       true if there was a gateway.
      */
-    static PBoolean GetNetworkInterface(PIPSocket::Address & addr);
+    static Address GetGatewayAddress(
+      unsigned version = 4  ///< IP version number
+    );
 
-#if P_HAS_RECVMSG
+    /**Get the name for the interface that is being used as the gateway,
+       that is, the interface that packets on the default route will be sent.
 
-    /**Set flag to capture destination address for incoming packets.
+       The string returned may be used in the Connect() function to open that
+       interface.
 
-       @return true if host is able to capture incoming address, else false.
-      */
-    PBoolean SetCaptureReceiveToAddress()
-    { if (!SetOption(IP_PKTINFO, 1, SOL_IP)) return false; catchReceiveToAddr = true; return true; }
+       Note that the driver does not need to be open for this function to work.
 
-    /**Return the interface address of the last incoming packet.
-      */
-    PIPSocket::Address GetLastReceiveToAddress() const
-    { return lastReceiveToAddr; }
-
-  protected:
-    void SetLastReceiveAddr(void * addr, int addrLen)
-    { if (addrLen == sizeof(in_addr)) lastReceiveToAddr = *(in_addr *)addr; }
-
-    PIPSocket::Address lastReceiveToAddr;
-
-#else
-
-    /**Set flag to capture interface address for incoming packets
-
-       @return true if host is able to capture incoming address, else false
-      */
-    PBoolean SetCaptureReceiveToAddress()
-    { return false; }
-
-    /**Return the interface address of the last incoming packet.
+       @return
+       String name of the gateway device, or empty string if there is none.
      */
-    PIPSocket::Address GetLastReceiveToAddress() const
-    { return PIPSocket::Address(); }
+    static PString GetGatewayInterface(
+      unsigned version = 4  ///< IP version number
+    );
 
-#endif
+    /**Get the IP address for the interface that is being used as the gateway,
+       that is, the interface that packets on the default route will be sent.
+
+       This Function can be used to Bind the Listener to only the default Packet
+       route in DHCP Environs.
+
+       Note that the driver does not need to be open for this function to work.
+
+       @return
+       The Local Interface IP Address for Gatway Access.
+     */
+    static Address GetGatewayInterfaceAddress(
+      unsigned version = 4  ///< IP version number
+    );
+
+    /**Get the interface address that will be used to reach the specified
+       remote address. Uses longest prefix match when multiple matching interfaces
+       are found.
+
+       @return
+       Network interface address.
+      */
+    static Address GetRouteInterfaceAddress(
+      Address remoteAddress    ///< Remote address to route
+    );
+
+    /// The types of QoS supported, based on IEEE P802.1p TrafficClass parameter
+    P_DECLARE_ENUM(QoSType,
+      BackgroundQoS,        ///< Reduced priority
+      BestEffortQoS,        ///< Effectively no QoS
+      ExcellentEffortQoS,   ///< Try a bit harder
+      CriticalQoS,          ///< Try really hard
+      VideoQoS,             ///< Video, < 100 ms latency and jitter
+      VoiceQoS,             ///< Voice, < 10 ms latency and jitter
+      ControlQoS            ///< Important stuff, prioritise over everything
+    );
+  /**@name Quality of Service
+     This describes in a platform and as protocol independent way as possible
+     the quality of service. How it produces, for example, DiffServ and RSVP
+     is up to the underlying operating system.
+    */
+  //@{
+    struct QoS
+    {
+      QoS(QoSType type = BestEffortQoS);
+      QoS(const PString & str);
+
+      QoSType m_type;
+      int     m_dscp; // If between 0 and 63, is used instead of default for QoSType.
+
+      AddressAndPort m_remote;
+
+      struct Flow {
+        Flow() { memset(this, 0, sizeof(*this)); }
+        unsigned m_maxBandwidth;    // bits/second, includes IP overhead
+        unsigned m_maxPacketSize;   // Bytes, includes IP overhead
+        unsigned m_maxLatency;      // Microseconds
+        unsigned m_maxJitter;       // Microseconds
+      } m_transmit, m_receive;
+
+      friend ostream & operator<<(ostream & strm, const PIPSocket::QoS & qos);
+      friend istream & operator>>(istream & strm, PIPSocket::QoS & qos);
+    };
+
+    /// Set the current Quality of Service
+    virtual bool SetQoS(
+      const QoS & qos   ///< New quality of service specification
+    );
+
+    /// Get the current Quality of Service
+    const QoS & GetQoS() const { return m_qos; }
+  //@}
+
+    virtual bool InternalGetLocalAddress(AddressAndPort & addrAndPort);
+    virtual bool InternalGetPeerAddress(AddressAndPort & addrAndPort);
+    virtual bool InternalListen(const Address & bind, unsigned queueSize, WORD port, Reusability reuse);
 
 // Include platform dependent part of class
 #ifdef _WIN32
@@ -732,52 +855,44 @@ class PIPSocket : public PSocket
 #else
 #include "unix/ptlib/ipsock.h"
 #endif
-};
-
-class PIPSocketAddressAndPort
-{
-  public:
-    PIPSocketAddressAndPort()
-      : m_port(0), m_separator(':')
-      { }
-
-    PIPSocketAddressAndPort(char separator)
-      : m_port(0), m_separator(separator)
-      { }
-
-    PIPSocketAddressAndPort(const PString & str, WORD defaultPort = 0, char separator = ':')
-      : m_port(defaultPort), m_separator(separator)
-      { Parse(str, defaultPort, m_separator); }
-
-    PBoolean Parse(const PString & str, WORD defaultPort = 0, char separator = ':');
-
-    PString AsString(char separator = 0) const
-      { return m_address.AsString() + (separator ? separator : m_separator) + PString(PString::Unsigned, m_port); }
-
-    void SetAddress(
-      const PIPSocket::Address & addr,
-      WORD port = 0
-    );
-    const PIPSocket::Address & GetAddress() const { return m_address; }
-    WORD GetPort() const { return m_port; }
-    void SetPort(
-      WORD port
-    ) { m_port = port; }
-
-    bool IsValid() const { return m_address.IsValid() && m_port != 0; }
-
-    friend ostream & operator<<(ostream & strm, const PIPSocketAddressAndPort & ap)
-    {
-      return strm << ap.m_address << ap.m_separator << ap.m_port;
-    }
 
   protected:
-    PIPSocket::Address m_address;
-    WORD               m_port;
-    char               m_separator;
+    QoS m_qos;
+
+    class sockaddr_wrapper
+    {
+      public:
+        sockaddr_wrapper();
+        sockaddr_wrapper(const AddressAndPort & ipPort);
+        sockaddr_wrapper(const Address & ip, WORD port);
+
+        sockaddr* operator->() const { return addr; }
+        operator sockaddr*()   const { return addr; }
+        socklen_t GetSize() const;
+
+        PIPSocket::Address GetIP() const;
+        WORD GetPort() const;
+
+      private:
+        void Construct(const Address & ip, WORD port);
+
+        sockaddr_storage storage;
+        union {
+          sockaddr_storage * ptr;
+          sockaddr         * addr;
+          sockaddr_in      * addr4;
+    #if P_HAS_IPV6
+          sockaddr_in6     * addr6;
+    #endif
+        };
+    };
 };
 
-typedef std::vector<PIPSocketAddressAndPort> PIPSocketAddressAndPortVector;
+typedef PIPSocket::Address        PIPAddress;
+typedef PIPSocket::AddressAndPort PIPAddressAndPort;
+typedef PIPSocket::AddressAndPort PIPSocketAddressAndPort;
+
+typedef std::vector<PIPSocket::AddressAndPort> PIPSocketAddressAndPortVector;
 
 
 #endif // PTLIB_IPSOCKET_H
