@@ -7,9 +7,9 @@
  *
  * Contributor(s): Snark at GnomeMeeting
  *
- * $Revision: 26520 $
- * $Author: rjongbloed $
- * $Date: 2011-10-04 23:54:38 -0500 (Tue, 04 Oct 2011) $
+ * $Revision: 28941 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-19 01:18:28 +1100 (Sat, 19 Jan 2013) $
  */
 
 #ifndef PTLIB_PLUGINMGR_H
@@ -78,7 +78,7 @@ class PPluginManager : public PObject
     PBoolean RegisterService (const PString & serviceName, const PString & serviceType, PPluginServiceDescriptor * descriptor);
 
     // Add a directory to the list of plugin directories (used by OPAL)
-    static void AddPluginDirs(PString dirs);
+    static bool AddPluginDirs(const PString & dirs);
 
     // Get the list of plugin directories
     static PStringArray GetPluginDirs();
@@ -117,8 +117,6 @@ class PPluginManager : public PObject
       const PNotifier & filterFunction
     );
 
-    static PString additionalPluginDirs;
-
   protected:
     void LoadPluginDirectory (const PDirectory & directory, const PStringList & suffixes);
     void CallNotifier(PDynaLink & dll, NotificationCode code);
@@ -151,7 +149,7 @@ class PPluginModuleManager : public PObject
     void LoadPluginDirectory(const PDirectory &directory)
     { if (pluginMgr != NULL) pluginMgr->LoadPluginDirectory(directory); }
 
-    virtual void OnLoadPlugin(PDynaLink & /*dll*/, INT /*code*/)
+    virtual void OnLoadPlugin(PDynaLink & /*dll*/, P_INT_PTR /*code*/)
     { }
 
     virtual PluginListType GetPluginList() const
@@ -172,6 +170,8 @@ class PPluginModuleManager : public PObject
 };
 
 
+#define PLUGIN_LOADER_STARTUP_NAME "PluginLoaderStartup"
+
 PFACTORY_LOAD(PluginLoaderStartup);
 
 
