@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24177 $
+ * $Revision: 28065 $
  * $Author: rjongbloed $
- * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
+ * $Date: 2012-07-19 21:01:53 +1000 (Thu, 19 Jul 2012) $
  */
 
 #ifndef PTLIB_SYNC_H
@@ -42,35 +42,55 @@
 #include <ptlib/contain.h>
 #include <ptlib/object.h>
 
+
+class PTimeInterval;
+
+
 class PSync : public PObject
 {
   public:
+    PSync() { }
+
   /**@name Operations */
   //@{
     /**Block until the synchronisation object is available.
      */
     virtual void Wait() = 0;
 
-    /**Signal that the synchronisation object is available.
+    /**Block, for a time, until the synchronisation object is available.
+
+       @return
+       true if lock is acquired, false if timed out
+     */
+    virtual PBoolean Wait(
+      const PTimeInterval & timeout // Amount of time to wait.
+    ) = 0;
+
+     /**Signal that the synchronisation object is available.
      */
     virtual void Signal() = 0;
   //@}
 
-#ifdef P_PTHREADS
-    PSync()
-      : lockerId(pthread_t(-1)) { }
-  protected:
-    pthread_t lockerId;
-#endif
+  private:
+    PSync(const PSync &) : PObject() { }
+    void operator=(const PSync &) { }
 };
 
+
+/// Synchronisation without really synchronising.
 class PSyncNULL : public PSync
 {
   public:
     virtual void Wait() { }
+    virtual PBoolean Wait(const PTimeInterval &) { return true; }
     virtual void Signal() { }
+
+  private:
+    PSyncNULL(const PSyncNULL &) : PSync() { }
+    void operator=(const PSyncNULL &) { }
 };
 
+
 /**This class waits for the semaphore on construction and automatically
    signals the semaphore on destruction. Any descendent of PSemaphore
    may be used.
