@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27275 $
+ * $Revision: 30263 $
  * $Author: rjongbloed $
- * $Date: 2012-03-22 16:51:22 -0500 (Thu, 22 Mar 2012) $
+ * $Date: 2013-08-07 16:37:36 +1000 (Wed, 07 Aug 2013) $
  */
  
 #ifndef PTLIB_SAFE_COLLECTION_H
@@ -36,6 +36,12 @@
 #endif
 
 
+#include <ptlib/syncthrd.h>
+
+
+class PTimer;
+
+
 /** This class defines a thread-safe object in a collection.
 
   This is part of a set of classes to solve the general problem of a
@@ -257,6 +263,12 @@ class PSafeObject : public PObject
        @return true if object may be deleted.
       */
     virtual bool GarbageCollection();
+
+    /**Get count of references to this object.
+       Note this returns the value outside of any mutexes, so it could change
+       at any moment. Care must be exercised in its use.
+      */
+    unsigned GetSafeReferenceCount() const { return safeReferenceCount; }
   //@}
 
   private:
@@ -367,6 +379,16 @@ class PSafeCollection : public PObject
     );
 
   public:
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream &strm   // Stream to print the object into.
+    ) const;
+
     /**Remove all objects in collection.
       */
     virtual void RemoveAll(
@@ -421,6 +443,7 @@ class PSafeCollection : public PObject
   protected:
     void CopySafeCollection(PCollection * other);
     void CopySafeDictionary(PAbstractDictionary * other);
+    bool SafeAddObject(PSafeObject * obj, PSafeObject * old);
     void SafeRemoveObject(PSafeObject * obj);
     PDECLARE_NOTIFIER(PTimer, PSafeCollection, DeleteObjectsTimeout);
 
@@ -429,7 +452,7 @@ class PSafeCollection : public PObject
     bool               deleteObjects;
     PList<PSafeObject> toBeRemoved;
     PMutex             removalMutex;
-    PTimer             deleteObjectsTimer;
+    PTimer           * m_deleteObjectsTimer;
 
   private:
     PSafeCollection(const PSafeCollection & other) : PObject(other) { }
@@ -526,6 +549,16 @@ class PSafePtrBase : public PObject
     virtual Comparison Compare(
       const PObject & obj   ///< Other instance to compare against
     ) const;
+
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream &strm   // Stream to print the object into.
+    ) const;
   //@}
 
   /**@name Operations */
@@ -538,6 +571,15 @@ class PSafePtrBase : public PObject
       */
     bool operator!() const { return currentObject == NULL; }
 
+    /**Return pointer to safe object.
+      */
+    PSafeObject * GetObject() const { return currentObject; }
+
+    /**Return pointer to safe object.
+      */
+    template <class T>
+    T * GetObjectAs() const { return dynamic_cast<T *>(currentObject); }
+
     /**Get the locking mode used by this pointer.
       */
     PSafetyMode GetSafetyMode() const { return lockMode; }
@@ -551,10 +593,6 @@ class PSafePtrBase : public PObject
     virtual PBoolean SetSafetyMode(
       PSafetyMode mode  ///< New locking mode
     );
-
-    /**Get the associated collection this pointer may be contained in.
-      */
-    const PSafeCollection * GetCollection() const { return collection; }
   //@}
 
     virtual void Assign(const PSafePtrBase & ptr);
@@ -856,15 +894,15 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
   //@{
     /**Return the physical pointer to the object.
       */
-    operator T*()    const { return  (T *)BaseClass::currentObject; }
+    operator T*()    const { return  dynamic_cast<T *>(BaseClass::currentObject); }
 
     /**Return the physical pointer to the object.
       */
-    T & operator*()  const { return *(T *)PAssertNULL(BaseClass::currentObject); }
+    T & operator*()  const { return *dynamic_cast<T *>(PAssertNULL(BaseClass::currentObject)); }
 
     /**Allow access to the physical object the pointer is pointing to.
       */
-    T * operator->() const { return  (T *)PAssertNULL(BaseClass::currentObject); }
+    T * operator->() const { return  dynamic_cast<T *>(PAssertNULL(BaseClass::currentObject)); }
 
     /**Post-increment the pointer.
        This requires that the pointer has been created with a PSafeCollection
@@ -872,7 +910,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator++(int)
       {
-        T * previous = (T *)BaseClass::currentObject;
+        T * previous = dynamic_cast<T *>(BaseClass::currentObject);
         BaseClass::Next();
         return previous;
       }
@@ -884,7 +922,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
     T * operator++()
       {
         BaseClass::Next();
-        return (T *)BaseClass::currentObject;
+        return dynamic_cast<T *>(BaseClass::currentObject);
       }
 
     /**Post-decrement the pointer.
@@ -893,7 +931,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator--(int)
       {
-        T * previous = (T *)BaseClass::currentObject;
+        T * previous = dynamic_cast<T *>(BaseClass::currentObject);
         BaseClass::Previous();
         return previous;
       }
@@ -905,7 +943,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
     T * operator--()
       {
         BaseClass::Previous();
-        return (T *)BaseClass::currentObject;
+        return dynamic_cast<T *>(BaseClass::currentObject);
       }
   //@}
 };
@@ -917,10 +955,8 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
 template <class Base, class Derived>
 PSafePtr<Derived> PSafePtrCast(const PSafePtr<Base> & oldPtr)
 {
-//  return PSafePtr<Derived>::DownCast<Base>(oldPtr);
     PSafePtr<Derived> newPtr;
-    Base * realPtr = oldPtr;
-    if (realPtr != NULL && PIsDescendant(realPtr, Derived))
+    if (dynamic_cast<Derived *>(oldPtr.GetObject()) != NULL)
       newPtr.Assign(oldPtr);
     return newPtr;
 }
@@ -938,7 +974,7 @@ PSafePtr<Derived> PSafePtrCast(const PSafePtr<Base> & oldPtr)
  */
 template <class Coll, class Base> class PSafeColl : public PSafeCollection
 {
-    PCLASSINFO(PSafeColl, PSafeCollection);
+    PCLASSINFO_WITH_CLONE(PSafeColl, PSafeCollection);
   public:
   /**@name Construction */
   //@{
@@ -984,8 +1020,7 @@ template <class Coll, class Base> class PSafeColl : public PSafeCollection
       PSafetyMode mode = PSafeReference   ///< Safety mode for returned locked PSafePtr
     ) {
         PWaitAndSignal mutex(collectionMutex);
-        if (PAssert(collection->GetObjectsIndex(obj) == P_MAX_INDEX, "Cannot insert safe object twice") &&
-            obj->SafeReference())
+        if (SafeAddObject(obj, NULL))
           return PSafePtr<Base>(*this, mode, collection->Append(obj));
         return NULL;
       }
@@ -1094,7 +1129,7 @@ template <class Base> class PSafeSortedList : public PSafeColl<PSortedList<Base>
  */
 template <class Coll, class Key, class Base> class PSafeDictionaryBase : public PSafeCollection
 {
-    PCLASSINFO(PSafeDictionaryBase, PSafeCollection);
+    PCLASSINFO_WITH_CLONE(PSafeDictionaryBase, PSafeCollection);
   public:
   /**@name Construction */
   //@{
@@ -1137,10 +1172,8 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
     virtual void SetAt(const Key & key, Base * obj)
       {
         collectionMutex.Wait();
-        SafeRemove(((Coll *)collection)->GetAt(key));
-        if (PAssert(collection->GetObjectsIndex(obj) == P_MAX_INDEX, "Cannot insert safe object twice") &&
-            obj->SafeReference())
-          ((Coll *)collection)->SetAt(key, obj);
+        if (SafeAddObject(obj, dynamic_cast<Coll &>(*collection).GetAt(key)))
+          dynamic_cast<Coll &>(*collection).SetAt(key, obj);
         collectionMutex.Signal();
       }
 
@@ -1156,7 +1189,7 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       const Key & key   ///< Key to fund object to delete
     ) {
         PWaitAndSignal mutex(collectionMutex);
-        return SafeRemove(((Coll *)collection)->GetAt(key));
+        return SafeRemove(dynamic_cast<Coll &>(*collection).GetAt(key));
       }
 
     /**Determine of the dictionary contains an entry for the key.
@@ -1165,7 +1198,7 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       const Key & key
     ) {
         PWaitAndSignal lock(collectionMutex);
-        return ((Coll *)collection)->Contains(key);
+        return dynamic_cast<Coll &>(*collection).Contains(key);
       }
 
     /**Get the instance in the collection of the index.
@@ -1188,9 +1221,9 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
     virtual PSafePtr<Base> FindWithLock(
       const Key & key,
       PSafetyMode mode = PSafeReadWrite
-    ) {
+    ) const {
         collectionMutex.Wait();
-        PSafePtr<Base> ptr(*this, PSafeReference, ((Coll *)collection)->GetAt(key));
+        PSafePtr<Base> ptr(*this, PSafeReference, dynamic_cast<Coll &>(*collection).GetAt(key));
         collectionMutex.Signal();
         ptr.SetSafetyMode(mode);
         return ptr;
@@ -1202,7 +1235,7 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
     {
       PArray<Key> keys;
       collectionMutex.Wait();
-      ((Coll *)collection)->AbstractGetKeys(keys);
+      dynamic_cast<Coll &>(*collection).AbstractGetKeys(keys);
       collectionMutex.Signal();
       return keys;
     }
@@ -1214,10 +1247,133 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
   See the PSafeObject class for more details. Especially in regard to
   enumeration of collections.
  */
-template <class Key, class Base> class PSafeDictionary : public PSafeDictionaryBase<PDictionary<Key, Base>, Key, Base>
+template <class K, class D>
+ class PSafeDictionary : public PSafeDictionaryBase<PDictionary<K, D>, K, D>
 {
   public:
-    typedef PSafePtr<Base> value_type;
+    typedef K key_type;
+    typedef D data_type;
+    typedef PSafePtr<D> value_type;
+    typedef PSafeDictionary<K, D> dict_type;
+
+  /**@name Iterators */
+  //@{
+    class iterator;
+    class const_iterator;
+    class iterator_base {
+      protected:
+        K * m_internal_first;  // Must be first two members
+        value_type m_internal_second;
+
+        const dict_type * m_dictionary;
+        PArray<K>   m_keys;
+        PINDEX      m_position;
+
+        iterator_base()
+          : m_internal_first(NULL)
+          , m_internal_second(NULL)
+          , m_dictionary(NULL)
+          , m_position(P_MAX_INDEX)
+        {
+        }
+
+        iterator_base(const dict_type * dict)
+          : m_dictionary(dict)
+          , m_keys(dict->GetKeys())
+        {
+          this->SetPosition(0);
+        }
+
+        iterator_base(const dict_type * dict, const K & key)
+          : m_dictionary(dict)
+          , m_keys(dict->GetKeys())
+        {
+          this->SetPosition(m_keys.GetValuesIndex(key));
+        }
+
+        bool SetPosition(PINDEX position)
+        {
+          if (position >= this->m_keys.GetSize()) {
+            this->m_position = P_MAX_INDEX;
+            this->m_internal_first = NULL;
+            this->m_internal_second.SetNULL();
+            return false;
+          }
+
+          this->m_position = position;
+          this->m_internal_first  = &this->m_keys[position];
+          this->m_internal_second = this->m_dictionary->FindWithLock(*this->m_internal_first, PSafeReference);
+          return this->m_internal_second == NULL;
+        }
+
+        void Next() { while (this->SetPosition(this->m_position+1)) { } }
+        void Prev() { while (this->SetPosition(this->m_position > 0 ? this->m_position+1 : P_MAX_INDEX)) { } }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->m_position == it.m_position; }
+        bool operator!=(const iterator_base & it) const { return this->m_position != it.m_position; }
+    };
+
+    class iterator_pair {
+      public:
+        const K & first;
+        value_type second;
+
+      private:
+        iterator_pair() : first(reinterpret_cast<const K &>(0)) { }
+    };
+
+    class iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair> {
+      protected:
+        iterator(dict_type * dict) : iterator_base(dict) { }
+        iterator(dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        iterator() { }
+
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
+
+        const iterator_pair * operator->() const { return  reinterpret_cast<const iterator_pair *>(this); }
+        const iterator_pair & operator* () const { return *reinterpret_cast<const iterator_pair *>(this); }
+
+      friend class PSafeDictionary<K, D>;
+    };
+
+    iterator begin() { return iterator(this); }
+    iterator end()   { return iterator(); }
+    iterator find(const K & key) { return iterator(this, key); }
+
+
+    class const_iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair> {
+      protected:
+        const_iterator(const dict_type * dict) : iterator_base(dict) { }
+        const_iterator(const dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        const_iterator() { }
+        const_iterator(const typename dict_type::iterator & it) : iterator_base(it) { }
+
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
+
+        const iterator_pair * operator->() const { return  reinterpret_cast<const iterator_pair *>(this); }
+        const iterator_pair & operator* () const { return *reinterpret_cast<const iterator_pair *>(this); }
+
+      friend class PSafeDictionary<K, D>;
+    };
+
+    const_iterator begin() const { return const_iterator(this); }
+    const_iterator end()   const { return const_iterator(); }
+    const_iterator find(const K & key) const { return const_iterator(this, key); }
+
+    void erase(const       iterator & it) { this->RemoveAt(it->first); }
+    void erase(const const_iterator & it) { this->RemoveAt(it->first); }
+  //@}
 };
 
 
