@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 25752 $
+ * $Revision: 30763 $
  * $Author: rjongbloed $
- * $Date: 2011-05-11 21:50:32 -0500 (Wed, 11 May 2011) $
+ * $Date: 2013-10-21 12:13:48 +1100 (Mon, 21 Oct 2013) $
  */
 
 
@@ -48,16 +48,13 @@ class PColourConverter;
 
 class PVideoFrameInfo : public PObject
 {
-  PCLASSINFO(PVideoFrameInfo, PObject);
-
+    PCLASSINFO(PVideoFrameInfo, PObject);
   public:
-    enum ResizeMode
-    {
-      eScale,
+    P_DECLARE_ENUM_EX(ResizeMode,eMaxResizeMode,
+      eScale,0,
       eCropCentre,
-      eCropTopLeft,
-      eMaxResizeMode
-    };
+      eCropTopLeft
+    );
     friend ostream & operator<<(ostream & strm, ResizeMode mode);
 
     enum StandardSizes {
@@ -86,6 +83,19 @@ class PVideoFrameInfo : public PObject
       ResizeMode      resizeMode = eScale
     );
 
+    /** Compare the two PVideoFrameInfo and return their relative rank.
+        This ranking is by the relative area of the frame resolution, and
+        frame rate if resolution equal. The final check for equality is on
+        the colourFormat. The SAR and resize mode take no part.
+
+       @return
+       <code>LessThan</code>, <code>EqualTo</code> or <code>GreaterThan</code>
+       according to the relative rank of the objects.
+     */
+    virtual Comparison Compare(
+      const PObject & obj   // Object to compare against.
+    ) const;
+
     /** Output the contents of the object to the stream. The exact output is
        dependent on the exact semantics of the descendent class. This is
        primarily used by the standard <code>#operator<<</code> function.
@@ -112,8 +122,8 @@ class PVideoFrameInfo : public PObject
        variable and returns true.
     */
     virtual PBoolean GetFrameSize(
-      unsigned & width,
-      unsigned & height
+      unsigned & width, ///< Current width of frame
+      unsigned & height ///< Current height of frame
     ) const;
 
     /** Get the width of the frame being used.
@@ -128,30 +138,33 @@ class PVideoFrameInfo : public PObject
     */
     virtual unsigned GetFrameHeight() const;
 
-    /**Set the sar size to be used.
+    /**Set the Storage Aspect Ratio size to be used.
 
        Default behaviour sets the sarWidth and sarHeight variables and
-       returns PTrue.
+       returns true.
     */
-    virtual PBoolean SetFrameSar(unsigned width, unsigned height);
+    virtual PBoolean SetFrameSar(
+      unsigned width,   ///< New SAR width of frame
+      unsigned height   ///< New SAR height of frame
+    );
 
-     /**Get the sar size being used.
+     /**Get the Storage Aspect Ratio size being used.
 
        Default behaviour returns the value of the sarWidth and sarHeight
-       variable and returns PTrue.
+       variable and returns true.
     */
     virtual PBoolean GetSarSize(
-      unsigned & width,
-      unsigned & height
+      unsigned & width, ///< Current SAR width of frame
+      unsigned & height ///< Current SAR height of frame
     ) const;
 
-    /** Get the width of the sar being used.
+    /** Get the width of the Storage Aspect Ratio being used.
 
         Default behaviour returns the value of the sarWidth variable
     */
     virtual unsigned GetSarWidth() const;
 
-    /** Get the height of the sar being used.
+    /** Get the height of the Storage Aspect Ratio being used.
 
         Default behaviour returns the value of the sarHeight variable
     */
@@ -262,55 +275,60 @@ class PVideoControlInfo : public PObject
   PCLASSINFO(PVideoControlInfo, PObject);
 
  public:
+    P_DECLARE_STREAMABLE_ENUM(Types,
+      Pan,
+      Tilt,
+      Zoom
+    );
 
-    typedef enum {
-      ControlPan,
-      ControlTilt,
-      ControlZoom
-    } InputControlType;
+    Types m_type;
+    long  m_min;
+    long  m_max;
+    long  m_step;
+    long  m_default;
+    long  m_flags;
+    long  m_current;
 
-    static PString AsString(const InputControlType & type);
+    PVideoControlInfo()
+      : m_type(EndTypes)
+      , m_min(0)
+      , m_max(0)
+      , m_step(0)
+      , m_default(0)
+      , m_flags(0)
+      , m_current(0)
+    { }
 
-    InputControlType type;
-    long             min;
-    long             max;
-    long             step;
-    long             def;
-    long             flags;
-    long             current;
+    bool IsValid() const { return m_max > 0 && m_min < m_max; }
 };
 
 
 /**This class defines a video Input device control (Camera controls PTZ)
 */
 
-class PVideoInputControl : public PVideoControlInfo
+class PVideoInputControl : public PObject
 {
-    PCLASSINFO(PVideoInputControl, PVideoControlInfo);
+    PCLASSINFO(PVideoInputControl, PObject);
 
 public:
   ~PVideoInputControl();
 
-  virtual PBoolean Pan(long value, bool absolute = false );
+  virtual PBoolean Pan (long value, bool absolute = false);
   virtual PBoolean Tilt(long value, bool absolute = false);
   virtual PBoolean Zoom(long value, bool absolute = false);
 
-  long GetPan();
-  long GetTilt();
-  long GetZoom();
+  virtual long GetPan()  const { return m_control[PVideoControlInfo::Pan ].m_current; }
+  virtual long GetTilt() const { return m_control[PVideoControlInfo::Tilt].m_current; }
+  virtual long GetZoom() const { return m_control[PVideoControlInfo::Zoom].m_current; }
 
-  void Reset();
-  void SetCurrentPosition(const InputControlType ctype, long current);
+  virtual void Reset();
 
-  typedef std::list<PVideoControlInfo> InputDeviceControls;
+  bool IsValid(PVideoControlInfo::Types type) const { return m_control[type].IsValid(); }
+  const PVideoControlInfo & GetControl(PVideoControlInfo::Types type) const { return m_control[type]; }
 
 protected:
-  PBoolean GetVideoControlInfo(const InputControlType ctype, PVideoControlInfo & control);
-  PBoolean GetDefaultPosition(const InputControlType ctype, long & def);
-  PBoolean GetCurrentPosition(const InputControlType ctype, long & current);
-
-  std::list<PVideoControlInfo> m_info;
-  PMutex ccmutex;
+  PVideoControlInfo m_control[PVideoControlInfo::NumTypes];
+  PMutex            m_mutex;
 
 };
 
@@ -396,6 +414,17 @@ class PVideoDevice : public PVideoFrameInfo
       */
     virtual PStringArray GetDeviceNames() const = 0;
 
+    struct Attributes {
+      Attributes();
+
+      int m_brightness;
+      int m_contrast;
+      int m_saturation;
+      int m_hue;
+      int m_gamma;
+      int m_exposure;
+    };
+
     struct OpenArgs {
       OpenArgs();
 
@@ -412,11 +441,7 @@ class PVideoDevice : public PVideoFrameInfo
       bool        convertSize;
       ResizeMode  resizeMode;
       bool        flip;
-      int         brightness;
-      int         whiteness;
-      int         contrast;
-      int         colour;
-      int         hue;
+      Attributes  m_attributes;
     };
 
     /**Open the device given the device name.
@@ -475,6 +500,10 @@ class PVideoDevice : public PVideoFrameInfo
     */
     virtual int GetNumChannels();
 
+    /**Get the names of video channels available on the device.
+    */
+    virtual PStringArray GetChannelNames();
+
     /**Set the video channel to be used on the device.
        The channel number is an integer from 0 to GetNumChannels()-1. The
        special value of -1 will find the first working channel number.
@@ -492,6 +521,15 @@ class PVideoDevice : public PVideoFrameInfo
     */
     virtual int GetChannel() const;
 
+    /**Set the frame info to be used, trying converters if available.
+
+       This function will simply call SetColourFormatConverter(),
+       SetFrameSizeConverter() and SetFrameRate().
+    */
+    virtual bool SetFrameInfoConverter(
+      const PVideoFrameInfo & info // New frame infofor device.
+    );
+
     /**Set the colour format to be used, trying converters if available.
 
        This function will set the colour format on the device to one that
@@ -568,7 +606,7 @@ class PVideoDevice : public PVideoFrameInfo
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns true.
+       then sets the converter sizes.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -576,19 +614,29 @@ class PVideoDevice : public PVideoFrameInfo
     );
 
     /**Get the frame size being used.
-
-       Default behaviour returns the value of the frameWidth and frameHeight
-       variable and returns true.
+       If a converter exists, the destination frame size of the converter
+       is returned, not the underlying physical device. If you wish the
+       physical device frame size use PVideoFrameInfo::GetFrameSize().
     */
     virtual PBoolean GetFrameSize(
       unsigned & width,
       unsigned & height
     ) const;
 
-    /**Get the maximum frame size in bytes.
+    /**Get the colour format being used.
+       If a converter exists, the destination colour format of the converter
+       is returned, not the underlying physical device. If you wish the
+       physical device colout format use PVideoFrameInfo::GetColourFormat().
+      */
+    virtual const PString& GetColourFormat() const;
 
-       Note a particular device may be able to provide variable length
-       frames (eg motion JPEG) so will be the maximum size of all frames.
+    /**Get the maximum frame size in bytes.
+       This gets the maximum of the physical device or the converted frame
+       size and colour format. It is guranteed to be enought memory space for
+       any video grab or output.
+
+       Note, a particular device may be able to provide variable length
+       frames (eg motion JPEG) so this will be the maximum size of all frames.
       */
     virtual PINDEX GetMaxFrameBytes() = 0;
 
@@ -602,62 +650,19 @@ class PVideoDevice : public PVideoFrameInfo
      */
     virtual PBoolean CanCaptureVideo() const = 0;
 
-    /**Get the brightness of the image. 0xffff-Very bright. -1 is unknown.
+    /**Get video attributes.
      */
-    virtual int GetBrightness();
-
-    /**Set brightness of the image. 0xffff-Very bright.
-     */
-    virtual PBoolean SetBrightness(unsigned newBrightness);
-
-
-    /**Get the whiteness of the image. 0xffff-Very white. -1 is unknown.
-     */
-    virtual int GetWhiteness();
-
-    /**Set whiteness of the image. 0xffff-Very white.
-     */
-    virtual PBoolean SetWhiteness(unsigned newWhiteness);
-
-
-    /**Get the colour of the image. 0xffff-lots of colour. -1 is unknown.
-     */
-    virtual int GetColour();
-
-    /**Set colour of the image. 0xffff-lots of colour.
-     */
-    virtual PBoolean SetColour(unsigned newColour);
-
-
-    /**Get the contrast of the image. 0xffff-High contrast. -1 is unknown.
-     */
-    virtual int GetContrast();
-
-    /**Set contrast of the image. 0xffff-High contrast.
-     */
-    virtual PBoolean SetContrast(unsigned newContrast);
-
-
-    /**Get the hue of the image. 0xffff-High hue. -1 is unknown.
-     */
-    virtual int GetHue();
-
-    /**Set hue of the image. 0xffff-High hue.
-     */
-    virtual PBoolean SetHue(unsigned newHue);
-    
-    
-    /**Return whiteness, brightness, colour, contrast and hue in one call.
-     */
-    virtual PBoolean GetParameters(
-      int *whiteness,
-      int *brightness,
-      int *colour,
-      int *contrast,
-      int *hue
+    virtual bool GetAttributes(
+      Attributes & attributes
     );
 
-    
+    /**Set video attributes.
+     */
+    virtual bool SetAttributes(
+      const Attributes & attributes
+    );
+
+
     /** Set VideoFormat and VideoChannel in one ioctl
      */
     virtual PBoolean SetVideoChannelFormat (
@@ -679,7 +684,7 @@ class PVideoDevice : public PVideoFrameInfo
   protected:
     PINDEX GetMaxFrameBytesConverted(PINDEX rawFrameBytes) const;
 
-    PString      deviceName;
+    PCaselessString deviceName;
     int          lastError;
     VideoFormat  videoFormat;
     int          channelNumber;
@@ -690,11 +695,18 @@ class PVideoDevice : public PVideoFrameInfo
     PColourConverter * converter;
     PBYTEArray         frameStore;
 
-    int          frameBrightness; // 16 bit entity, -1 is no value
-    int          frameWhiteness;
-    int          frameContrast;
-    int          frameColour;
-    int          frameHue;
+  private:
+    P_REMOVE_VIRTUAL(int, GetBrightness(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetBrightness(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetWhiteness(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetWhiteness(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetColour(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetColour(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetContrast(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetContrast(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetHue(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetHue(unsigned), false);
+    P_REMOVE_VIRTUAL(PBoolean, GetParameters(int *, int *, int *, int *, int *), false);
 };
 
 
@@ -785,32 +797,32 @@ class PVideoOutputDevice : public PVideoDevice
     /**Set a section of the output frame buffer.
       */
     virtual PBoolean SetFrameData(
-      unsigned x,
-      unsigned y,
-      unsigned width,
-      unsigned height,
-      const BYTE * data,
-      PBoolean endFrame = true
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame = true  ///< Indicate no more data for this video frame
     ) = 0;
     virtual PBoolean SetFrameData(
-      unsigned x,
-      unsigned y,
-      unsigned width,
-      unsigned height,
-      const BYTE * data,
-      PBoolean endFrame,
-      unsigned flags
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame,        ///< Indicate no more data for this video frame
+      bool & keyFrameNeeded     ///< Indicates bad video and a new key frame is required
     );
     virtual PBoolean SetFrameData(
-      unsigned x,
-      unsigned y,
-      unsigned width,
-      unsigned height,
-      unsigned sarwidth,
-      unsigned sarheight,
-      const BYTE * data,
-      PBoolean endFrame,
-      unsigned flags,
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      unsigned sarWidth,        ///< Aspect ratio width of area in frame where data is put
+      unsigned sarHeight,       ///< Aspect ratio height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame,        ///< Indicate no more data for this video frame
+      bool & keyFrameNeeded,    ///< Indicates bad video and a new key frame is required
       const void * mark
     );
 
@@ -1103,7 +1115,8 @@ class PVideoInputDevice : public PVideoDevice
     virtual PBoolean GetFrameData(
       BYTE * buffer,                 ///< Buffer to receive frame
       PINDEX * bytesReturned,        ///< Optional bytes returned.
-      unsigned int & flags           ///< optional flags returned
+      bool & keyFrame         /**< On input, forces generation of key frame,
+                                   On return indicates key frame generated */
     );
     virtual PBoolean GetFrameData(
       BYTE * buffer,                 ///< Buffer to receive frame
@@ -1115,7 +1128,8 @@ class PVideoInputDevice : public PVideoDevice
     virtual PBoolean GetFrameDataNoDelay(
       BYTE * buffer,                 ///< Buffer to receive frame
       PINDEX * bytesReturned,       ///< Optional bytes returned.
-      unsigned int & flags           ///< optional flags returned
+      bool & keyFrame         /**< On input, forces generation of key frame,
+                                   On return indicates key frame generated */
     );
     virtual PBoolean GetFrameDataNoDelay(
       BYTE * buffer,                 ///< Buffer to receive frame
@@ -1136,7 +1150,7 @@ class PVideoInputDevice : public PVideoDevice
 
        In video-mode the camera continuously sends new frames.
 
-       The default implementation does nothing but returns PFalse.
+       The default implementation does nothing but returns false.
       */
     virtual bool SetCaptureMode(unsigned mode);
 
@@ -1144,6 +1158,9 @@ class PVideoInputDevice : public PVideoDevice
        A return value of -1 indicates an error or the mode is not supported.
     */
     virtual int GetCaptureMode() const;
+
+    P_REMOVE_VIRTUAL(PBoolean, GetFrameData(BYTE *, PINDEX *, unsigned &), false);
+    P_REMOVE_VIRTUAL(PBoolean, GetFrameDataNoDelay(BYTE *, PINDEX *, unsigned &), false);
 };
 
 
@@ -1165,9 +1182,20 @@ template <class className> class PVideoInputPluginServiceDescriptor : public PDe
   static PVideoInputPluginServiceDescriptor<PVideoInputDevice_##name> PVideoInputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoInputDevice, &PVideoInputDevice_##name##_descriptor)
 
+
+#define P_FAKE_VIDEO_DRIVER         "FakeVideo"
+#define P_FAKE_VIDEO_PREFIX         "Fake/"
+#define P_FAKE_VIDEO_MOVING_BLOCKS  P_FAKE_VIDEO_PREFIX"MovingBlocks"
+#define P_FAKE_VIDEO_MOVING_LINE    P_FAKE_VIDEO_PREFIX"MovingLine"
+#define P_FAKE_VIDEO_BOUNCING_BOXES P_FAKE_VIDEO_PREFIX"BouncingBoxes"
+#define P_FAKE_VIDEO_SOLID_COLOUR   P_FAKE_VIDEO_PREFIX"SolidColour"
+#define P_FAKE_VIDEO_TEXT           P_FAKE_VIDEO_PREFIX"Text"
+#define P_FAKE_VIDEO_NTSC           P_FAKE_VIDEO_PREFIX"NTSCTest"
+
 PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
 
 #ifdef P_APPSHARE
+  #define P_APPLICATION_VIDEO_DRIVER "Application"
   PPLUGIN_STATIC_LOAD(Application, PVideoInputDevice);
 #endif
 
@@ -1176,10 +1204,17 @@ PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
 #endif
 
 #if P_VIDFILE
+  #define P_VIDEO_FILE_DRIVER "YUVFile"
   PPLUGIN_STATIC_LOAD(YUVFile, PVideoInputDevice);
 #endif
 
+#ifdef WIN32
+  #define P_VIDEO_FOR_WINDOWS_DRIVER "VideoForWindows"
+  PPLUGIN_STATIC_LOAD(VideoForWindows, PVideoInputDevice);
+#endif
+
 #ifdef P_DIRECTSHOW
+  #define P_DIRECT_SHOW_DRIVER "DirectShow"
   PPLUGIN_STATIC_LOAD(DirectShow, PVideoInputDevice);
 #endif
 
@@ -1200,11 +1235,27 @@ template <class className> class PVideoOutputPluginServiceDescriptor : public PD
   static PVideoOutputPluginServiceDescriptor<PVideoOutputDevice_##name> PVideoOutputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoOutputDevice, &PVideoOutputDevice_##name##_descriptor)
 
+#define P_NULL_VIDEO_DRIVER "NULLOutput"
+#define P_NULL_VIDEO_DEVICE "Null Video Out"
+PPLUGIN_STATIC_LOAD(NULLOutput, PVideoOutputDevice);
+
+
+#if P_VFW_CAPTURE
 #if _WIN32
+  #define P_MSWIN_VIDEO_DRIVER "Window"
+  #define P_MSWIN_VIDEO_PREFIX "MSWIN"
+  #define P_MSWIN_VIDEO_DEVICE(x,y,width,height) P_MSWIN_VIDEO_PREFIX " X=" x " Y=" y " WIDTH=" width " HEIGHT=" height
   PPLUGIN_STATIC_LOAD(Window, PVideoOutputDevice);
 #endif
+#endif
 
 #if P_SDL
+  #if P_MACOSX
+    #include <SDL_main.h>
+  #endif
+  #define P_SDL_VIDEO_DRIVER "SDL"
+  #define P_SDL_VIDEO_PREFIX "SDL"
+  #define P_SDL_VIDEO_DEVICE(x,y,width,height) P_SDL_VIDEO_PREFIX " X=" x " Y=" y " WIDTH=" width " HEIGHT=" height
   PPLUGIN_STATIC_LOAD(SDL, PVideoOutputDevice);
 #endif
 
